@@ -11,7 +11,6 @@ use crate::serialization::submessage::SubmessageBody;
 
 use crate::dds::reader::Reader;
 use crate::structure::guid::EntityId;
-use crate::messages::submessages::submessages::AckNack;
 
 #[cfg(test)] use crate::dds::ddsdata::DDSData;
 #[cfg(test)] use crate::structure::cache_change::CacheChange;
@@ -35,7 +34,7 @@ const RTPS_MESSAGE_HEADER_SIZE: usize = 20;
 pub(crate) struct MessageReceiver {
   pub available_readers: BTreeMap<EntityId,Reader>,
   // GuidPrefix sent in this channel needs to be RTPSMessage source_guid_prefix. Writer needs this to locate RTPSReaderProxy if negative acknack.
-  acknack_sender: mio_channel::SyncSender<(GuidPrefix, AckNack)>,
+  acknack_sender: mio_channel::SyncSender<(GuidPrefix, AckSubmessage)>,
 
   own_guid_prefix: GuidPrefix,
   pub source_version: ProtocolVersion,
@@ -53,7 +52,7 @@ pub(crate) struct MessageReceiver {
 impl MessageReceiver {
   pub fn new(
     participant_guid_prefix: GuidPrefix,
-    acknack_sender: mio_channel::SyncSender<(GuidPrefix, AckNack)>,
+    acknack_sender: mio_channel::SyncSender<(GuidPrefix, AckSubmessage)>,
   ) -> MessageReceiver {
     // could be passed in as a parameter
     let locator_kind = LocatorKind::LOCATOR_KIND_UDPv4;
@@ -219,7 +218,7 @@ impl MessageReceiver {
       EntitySubmessage::Data(data, data_flags) => {
         // If reader_id == ENTITYID_UNKNOWN, message should be sent to all matched readers
         if data.reader_id == EntityId::ENTITYID_UNKNOWN {
-          trace!("handle_entity_submessage DATA from unknown. writer_id = {:?}", &data.writer_id);
+          trace!("handle_entity_submessage DATA for unknown reader. writer_id = {:?}", &data.writer_id);
           for reader in self
             .available_readers
             .values_mut()
@@ -233,13 +232,14 @@ impl MessageReceiver {
                             )
                       )
           {
-            debug!("handle_entity_submessage DATA from unknown handling in {:?}",&reader);
+            debug!("handle_entity_submessage DATA for unknown reader. Handling in {:?}",&reader);
             reader.handle_data_msg(data.clone(), data_flags, mr_state.clone());
           }
+        } else if let Some(target_reader) = self.get_reader_mut(data.reader_id) {
+          target_reader.handle_data_msg(data, data_flags, mr_state);
         } else {
-          if let Some(target_reader) = self.get_reader_mut(data.reader_id) {
-            target_reader.handle_data_msg(data, data_flags, mr_state);
-          }
+          // We have no such reader present
+          debug!("DATA for reader {:?}, but it does not exist.", data.reader_id)
         }
       }
       EntitySubmessage::Heartbeat(heartbeat, flags) => {
@@ -256,14 +256,14 @@ impl MessageReceiver {
               mr_state.clone(),
             );
           }
+        } else if let Some(target_reader) = self.get_reader_mut(heartbeat.reader_id) {
+          target_reader.handle_heartbeat_msg(
+            heartbeat,
+            flags.contains(HEARTBEAT_Flags::Final),
+            mr_state,
+          );
         } else {
-          if let Some(target_reader) = self.get_reader_mut(heartbeat.reader_id) {
-            target_reader.handle_heartbeat_msg(
-              heartbeat,
-              flags.contains(HEARTBEAT_Flags::Final),
-              mr_state,
-            );
-          }
+          debug!("HEARTBEAT for reader {:?}, but it does not exist.", heartbeat.reader_id)
         }
       }
       EntitySubmessage::Gap(gap, _flags) => {
@@ -272,14 +272,15 @@ impl MessageReceiver {
         }
       }
       EntitySubmessage::AckNack(acknack, _) => {
-        match self.acknack_sender.send((self.source_guid_prefix, acknack)) {
+        match self.acknack_sender.send((self.source_guid_prefix, 
+            AckSubmessage::AckNack_Variant(acknack))) {
           Ok(_) => (),
           Err(e) => warn!("Failed to send AckNack. {:?}", e),
         }
       }
-      EntitySubmessage::DataFrag(datafrag, _) => {
+      EntitySubmessage::DataFrag(datafrag, flags) => {
         if let Some(target_reader) = self.get_reader_mut(datafrag.reader_id) {
-          target_reader.handle_datafrag_msg(datafrag, mr_state);
+          target_reader.handle_datafrag_msg(datafrag, flags, mr_state, );
         }
       }
       EntitySubmessage::HeartbeatFrag(heartbeatfrag, _flags) => {
@@ -292,10 +293,10 @@ impl MessageReceiver {
           {
             reader.handle_heartbeatfrag_msg(heartbeatfrag.clone(), mr_state.clone());
           }
+        } else if let Some(target_reader) = self.get_reader_mut(heartbeatfrag.reader_id) {
+          target_reader.handle_heartbeatfrag_msg(heartbeatfrag, mr_state); 
         } else {
-          if let Some(target_reader) = self.get_reader_mut(heartbeatfrag.reader_id) {
-            target_reader.handle_heartbeatfrag_msg(heartbeatfrag, mr_state);
-          }
+          debug!("HEARTBEATFRAG for reader {:?}, but it does not exist.", heartbeatfrag.reader_id)
         }
       }
       EntitySubmessage::NackFrag(_, _) => {}
@@ -384,6 +385,9 @@ use super::*;
     dds::writer::WriterCommand, messages::header::Header,
     dds::with_key::datareader::ReaderCommand,
   };
+  use crate::dds::reader::ReaderIngredients;
+  use crate::dds::writer::WriterIngredients;
+  use crate::network::udp_sender::UDPSender;
   use crate::dds::statusevents::DataReaderStatus;
   use crate::speedy::{Writable, Readable};
   use crate::serialization::cdr_deserializer::deserialize_from_little_endian;
@@ -395,6 +399,7 @@ use super::*;
   use mio_extras::channel as mio_channel;
   use crate::structure::dds_cache::DDSCache;
   use std::sync::{RwLock, Arc};
+  use std::rc::Rc;
 
   use crate::structure::topic_kind::TopicKind;
   use crate::structure::guid::EntityKind;
@@ -424,7 +429,7 @@ use super::*;
     ]);
 
     let (acknack_sender, _acknack_reciever) =
-      mio_channel::sync_channel::<(GuidPrefix, AckNack)>(10);
+      mio_channel::sync_channel::<(GuidPrefix, AckSubmessage)>(10);
     let mut message_receiver = MessageReceiver::new(guiPrefix, acknack_sender);
 
     let entity = EntityId::createCustomEntityID([0, 0, 0], EntityKind::READER_WITH_KEY_USER_DEFINED);
@@ -442,15 +447,17 @@ use super::*;
       TopicKind::NoKey,
       TypeDesc::new("testi"),
     );
-    let new_reader = Reader::new(
-      new_guid,
-      send,
+    let reader_ing = ReaderIngredients {
+      guid: new_guid,
+      notification_sender: send,
       status_sender,
-      dds_cache,
-      "test".to_string(),
-      QosPolicies::qos_none(), // TODO: Check intended qos
-      reader_command_receiver,
-    );
+      topic_name: "test".to_string(),
+      qos_policy: QosPolicies::qos_none(),
+      data_reader_command_receiver: reader_command_receiver,
+    };
+
+    let new_reader = Reader::new(reader_ing, dds_cache,
+      Rc::new(UDPSender::new_with_random_port().unwrap()));
 
     // Skip for now+
     //new_reader.matched_writer_add(remote_writer_guid, mr_state);
@@ -491,13 +498,19 @@ use super::*;
     let (_dwcc_upload, hccc_download) = mio_channel::channel::<WriterCommand>();
     let (status_sender, _status_receiver) = mio_channel::sync_channel(10);
 
-    let mut _writerObject = Writer::new(
-      GUID::new_with_prefix_and_id(guiPrefix, EntityId::createCustomEntityID([0, 0, 2], EntityKind::WRITER_WITH_KEY_USER_DEFINED)),
-      hccc_download,
-      Arc::new(RwLock::new(DDSCache::new())),
-      String::from("topicName1"),
-      QosPolicies::qos_none(),
+    let writer_ing = WriterIngredients {
+      guid: GUID::new_with_prefix_and_id(guiPrefix, 
+          EntityId::createCustomEntityID([0, 0, 2], EntityKind::WRITER_WITH_KEY_USER_DEFINED)),
+      writer_command_receiver: hccc_download,
+      topic_name: String::from("topicName1"),
+      qos_policies: QosPolicies::qos_none(),
       status_sender,
+    };
+
+    let mut _writerObject = Writer::new(
+      writer_ing,
+      Arc::new(RwLock::new(DDSCache::new())),
+      Rc::new(UDPSender::new_with_random_port().unwrap())
     );
     let mut change = message_receiver.get_reader_and_history_cache_change_object(
       new_guid.entityId,
@@ -531,7 +544,7 @@ use super::*;
 
     let guid_new = GUID::default();
     let (acknack_sender, _acknack_reciever) =
-      mio_channel::sync_channel::<(GuidPrefix, AckNack)>(10);
+      mio_channel::sync_channel::<(GuidPrefix, AckSubmessage)>(10);
     let mut message_receiver = MessageReceiver::new(guid_new.guidPrefix, acknack_sender);
 
     message_receiver.handle_received_packet(udp_bits1);
