@@ -1,4 +1,4 @@
-use mio::Token;
+//use mio::Token;
 use mio_extras::channel as mio_channel;
 #[allow(unused_imports)]
 use log::{error, debug, info, warn, trace};
@@ -20,7 +20,7 @@ use crate::{
 };
 
 use crate::dds::{
-  dp_event_loop::DPEventLoop, reader::*, writer::Writer, pubsub::*, topic::*, typedesc::*, qos::*,
+  dp_event_loop::DPEventLoop, reader::*, writer::*, pubsub::*, topic::*, typedesc::*, qos::*,
   values::result::*,
 };
 
@@ -52,7 +52,11 @@ impl DomainParticipant {
   /// ```
   pub fn new(domain_id: u16) -> Result<DomainParticipant> {
     trace!("DomainParticipant construct start");
+
+    // Discovery join channel is used to just send a join handle into the inner participant,
+    // so its .drop() can wait until discovery has had a chance to stop.
     let (djh_sender, djh_receiver) = mio_channel::channel();
+
     let mut dpd = DomainParticipant_Disc::new(domain_id, djh_receiver)?;
 
     let discovery_updated_sender = match dpd.discovery_updated_sender.take() {
@@ -74,18 +78,26 @@ impl DomainParticipant {
     let (discovery_started_sender, discovery_started_receiver) =
       std::sync::mpsc::channel::<Result<()>>();
 
-    let discovery = Discovery::new(
-      dp.weak_clone(),
-      dp.discovery_db(),
-      discovery_started_sender,
-      discovery_updated_sender,
-      discovery_command_receiver,
-    );
-
+    // Construct and start background thread
+    let dp_clone = dp.weak_clone();
+    let disc_db_clone = dp.discovery_db();
     let discovery_handle = thread::Builder::new()
       .name("RustDDS discovery thread".to_string())
-      .spawn(move || Discovery::discovery_event_loop(discovery))?;
-    djh_sender.send(discovery_handle).unwrap_or(());
+      .spawn(move || { 
+          match Discovery::new(
+              dp_clone,
+              disc_db_clone,
+              discovery_started_sender,
+              discovery_updated_sender,
+              discovery_command_receiver,
+            ) {
+            Ok(mut discovery) => // run the event loop
+              discovery.discovery_event_loop(),
+            Err(_) => (),
+          }
+        })?;
+
+    djh_sender.send(discovery_handle).unwrap_or(()); // send join handle to inner participant
 
     debug!("Waiting for discovery to start"); // blocking until discovery answers
     match discovery_started_receiver.recv_timeout(Duration::from_secs(10)) {
@@ -286,14 +298,14 @@ impl DomainParticipantWeak {
 
   pub fn create_publisher(&self, qos: &QosPolicies) -> Result<Publisher> {
     match self.dpi.upgrade() {
-      Some(dpi) => dpi.lock().unwrap().create_publisher(&self, qos),
+      Some(dpi) => dpi.lock().unwrap().create_publisher(self, qos),
       None => Err(Error::OutOfResources),
     }
   }
 
-  pub fn create_subscriber<'a>(&self, qos: &QosPolicies) -> Result<Subscriber> {
+  pub fn create_subscriber(&self, qos: &QosPolicies) -> Result<Subscriber> {
     match self.dpi.upgrade() {
-      Some(dpi) => dpi.lock().unwrap().create_subscriber(&self, qos),
+      Some(dpi) => dpi.lock().unwrap().create_subscriber(self, qos),
       None => Err(Error::OutOfResources),
     }
   }
@@ -309,14 +321,14 @@ impl DomainParticipantWeak {
       Some(dpi) => dpi
         .lock()
         .unwrap()
-        .create_topic(&self, name, type_desc, qos, topic_kind),
+        .create_topic(self, name, type_desc, qos, topic_kind),
       None => Err(Error::LockPoisoned),
     }
   }
 
   pub fn find_topic(&self, name: &str, timeout: Duration) -> Result<Option<Topic>> {
     match self.dpi.upgrade() {
-      Some(dpi) => dpi.lock().unwrap().find_topic(&self, name, timeout),
+      Some(dpi) => dpi.lock().unwrap().find_topic(self, name, timeout),
       None => Err(Error::LockPoisoned),
     }
   }
@@ -343,10 +355,8 @@ impl DomainParticipantWeak {
   }
 
   pub fn upgrade(self) -> Option<DomainParticipant> {
-    match self.dpi.upgrade() {
-      Some(d) => Some(DomainParticipant { dpi: d }),
-      None => None,
-    }
+    self.dpi.upgrade()
+      .map( |d| DomainParticipant { dpi: d } )
   }
 } // end impl
 
@@ -398,10 +408,10 @@ impl DomainParticipant_Disc {
       .dpi
       .lock()
       .unwrap()
-      .create_publisher(&dp, qos, self.discovery_command_channel.clone())
+      .create_publisher(dp, qos, self.discovery_command_channel.clone())
   }
 
-  pub fn create_subscriber<'a>(
+  pub fn create_subscriber(
     &self,
     dp: &DomainParticipantWeak,
     qos: &QosPolicies,
@@ -410,7 +420,7 @@ impl DomainParticipant_Disc {
       .dpi
       .lock()
       .unwrap()
-      .create_subscriber(&dp, qos, self.discovery_command_channel.clone())
+      .create_subscriber(dp, qos, self.discovery_command_channel.clone())
   }
 
   pub fn create_topic(
@@ -426,7 +436,7 @@ impl DomainParticipant_Disc {
       .dpi
       .lock()
       .unwrap()
-      .create_topic(&dp, name, type_desc, qos, topic_kind)
+      .create_topic(dp, name, type_desc, qos, topic_kind)
   }
 
   pub fn find_topic(
@@ -435,7 +445,7 @@ impl DomainParticipant_Disc {
     name: &str,
     timeout: Duration,
   ) -> Result<Option<Topic>> {
-    self.dpi.lock().unwrap().find_topic(&dp, name, timeout)
+    self.dpi.lock().unwrap().find_topic(dp, name, timeout)
   }
 
   pub fn domain_id(&self) -> u16 {
@@ -451,7 +461,7 @@ impl DomainParticipant_Disc {
   }
 
   pub(crate) fn get_dds_cache(&self) -> Arc<RwLock<DDSCache>> {
-    return self.dpi.lock().unwrap().get_dds_cache();
+    self.dpi.lock().unwrap().get_dds_cache()
   }
 
   pub(crate) fn discovery_db(&self) -> Arc<RwLock<DiscoveryDB>> {
@@ -500,10 +510,9 @@ pub(crate) struct DomainParticipant_Inner {
   participant_id: u16,
 
   my_guid: GUID,
-  reader_binds: HashMap<Token, mio_channel::Receiver<(Token, Reader)>>,
 
   // Adding Readers
-  sender_add_reader: mio_channel::SyncSender<Reader>,
+  sender_add_reader: mio_channel::SyncSender<ReaderIngredients>,
   sender_remove_reader: mio_channel::SyncSender<GUID>,
 
   // Adding DataReaders
@@ -516,7 +525,7 @@ pub(crate) struct DomainParticipant_Inner {
   // out of the struct (take) in order to .join() on the handle.
 
   // Writers
-  add_writer_sender: mio_channel::SyncSender<Writer>,
+  add_writer_sender: mio_channel::SyncSender<WriterIngredients>,
   remove_writer_sender: mio_channel::SyncSender<GUID>,
 
   dds_cache: Arc<RwLock<DDSCache>>,
@@ -530,11 +539,10 @@ impl Drop for DomainParticipant_Inner {
     // ev_loop_thread anyways
     match self.stop_poll_sender.send(()) {
       Ok(_) => (),
-      _ => return (),
+      _ => return,
     };
 
     debug!("Waiting for dp_event_loop join");
-
     match self.ev_loop_handle.take() {
       Some(join_handle) => {
         join_handle
@@ -545,7 +553,6 @@ impl Drop for DomainParticipant_Inner {
         error!("Someone managed to steal dp_event_loop join handle from DomainParticipant_Inner.");
       }
     }
-
     debug!("Joined dp_event_loop");
   }
 }
@@ -621,11 +628,11 @@ impl DomainParticipant_Inner {
     listeners.insert(USER_TRAFFIC_LISTENER_TOKEN, user_traffic_listener);
 
     // Adding readers
-    let (sender_add_reader, receiver_add_reader) = mio_channel::sync_channel::<Reader>(100);
+    let (sender_add_reader, receiver_add_reader) = mio_channel::sync_channel::<ReaderIngredients>(100);
     let (sender_remove_reader, receiver_remove_reader) = mio_channel::sync_channel::<GUID>(10);
 
     // Writers
-    let (add_writer_sender, add_writer_receiver) = mio_channel::sync_channel::<Writer>(10);
+    let (add_writer_sender, add_writer_receiver) = mio_channel::sync_channel::<WriterIngredients>(10);
     let (remove_writer_sender, remove_writer_receiver) = mio_channel::sync_channel::<GUID>(10);
 
     let new_guid = GUID::new_particiapnt_guid();
@@ -635,41 +642,45 @@ impl DomainParticipant_Inner {
       participant_id,
     };
 
-    let a_r_cache = Arc::new(RwLock::new(DDSCache::new()));
+    let dds_cache = Arc::new(RwLock::new(DDSCache::new()));
 
     let discovery_db = Arc::new(RwLock::new(DiscoveryDB::new(new_guid)));
 
     let (stop_poll_sender, stop_poll_receiver) = mio_channel::channel::<()>();
 
-    let ev_wrapper = DPEventLoop::new(
-      domain_info,
-      listeners,
-      a_r_cache.clone(),
-      discovery_db.clone(),
-      new_guid.guidPrefix,
-      TokenReceiverPair {
-        token: ADD_READER_TOKEN,
-        receiver: receiver_add_reader,
-      },
-      TokenReceiverPair {
-        token: REMOVE_READER_TOKEN,
-        receiver: receiver_remove_reader,
-      },
-      TokenReceiverPair {
-        token: ADD_WRITER_TOKEN,
-        receiver: add_writer_receiver,
-      },
-      TokenReceiverPair {
-        token: REMOVE_WRITER_TOKEN,
-        receiver: remove_writer_receiver,
-      },
-      stop_poll_receiver,
-      discovery_update_notification_receiver,
-    );
     // Launch the background thread for DomainParticipant
+    let dds_cache_clone = dds_cache.clone();
+    let disc_db_clone = discovery_db.clone();
     let ev_loop_handle = thread::Builder::new()
-      .name("RustDDS Participant event loop".to_string())
-      .spawn(move || ev_wrapper.event_loop())?;
+      .name(format!("RustDDS Participant {} event loop",participant_id))
+      .spawn(move || { 
+        let dp_event_loop = DPEventLoop::new(
+          domain_info,
+          listeners,
+          dds_cache_clone,
+          disc_db_clone,
+          new_guid.guidPrefix,
+          TokenReceiverPair {
+            token: ADD_READER_TOKEN,
+            receiver: receiver_add_reader,
+          },
+          TokenReceiverPair {
+            token: REMOVE_READER_TOKEN,
+            receiver: receiver_remove_reader,
+          },
+          TokenReceiverPair {
+            token: ADD_WRITER_TOKEN,
+            receiver: add_writer_receiver,
+          },
+          TokenReceiverPair {
+            token: REMOVE_WRITER_TOKEN,
+            receiver: remove_writer_receiver,
+          },
+          stop_poll_receiver,
+          discovery_update_notification_receiver,
+        );
+        dp_event_loop.event_loop()
+      })?;
 
     info!("New DomainParticipant_Inner: domain_id={:?} participant_id={:?} GUID={:?}",
       domain_id, participant_id, new_guid);
@@ -677,8 +688,6 @@ impl DomainParticipant_Inner {
       domain_id,
       participant_id,
       my_guid: new_guid,
-      reader_binds: HashMap::new(),
-      // ddscache: a_r_cache,
       // Adding readers
       sender_add_reader,
       sender_remove_reader,
@@ -689,7 +698,7 @@ impl DomainParticipant_Inner {
       ev_loop_handle: Some(ev_loop_handle),
       add_writer_sender,
       remove_writer_sender,
-      dds_cache: Arc::new(RwLock::new(DDSCache::new())),
+      dds_cache,
       discovery_db,
     })
   }
@@ -698,7 +707,7 @@ impl DomainParticipant_Inner {
     self.dds_cache.clone()
   }
 
-  pub fn add_reader(&self, reader: Reader) {
+  pub fn add_reader(&self, reader: ReaderIngredients) {
     self.sender_add_reader.send(reader).unwrap();
   }
 
@@ -762,7 +771,7 @@ impl DomainParticipant_Inner {
       domain_participant_weak,
       name.to_string(),
       TypeDesc::new(type_desc),
-      &qos,
+      qos,
       topic_kind,
     );
     Ok(topic)
@@ -826,7 +835,7 @@ impl DomainParticipant_Inner {
 
   // The following methods are not for application use.
 
-  pub(crate) fn get_add_reader_sender(&self) -> mio_channel::SyncSender<Reader> {
+  pub(crate) fn get_add_reader_sender(&self) -> mio_channel::SyncSender<ReaderIngredients> {
     self.sender_add_reader.clone()
   }
 
@@ -834,7 +843,7 @@ impl DomainParticipant_Inner {
     self.sender_remove_reader.clone()
   }
 
-  pub(crate) fn get_add_writer_sender(&self) -> mio_channel::SyncSender<Writer> {
+  pub(crate) fn get_add_writer_sender(&self) -> mio_channel::SyncSender<WriterIngredients> {
     self.add_writer_sender.clone()
   }
 
@@ -856,7 +865,7 @@ impl DomainParticipant_Inner {
       Err(e) => panic!("DiscoveryDB is poisoned. {:?}", e),
     };
 
-    db.get_all_topics().map(|p| p.clone()).collect()
+    db.get_all_topics().cloned().collect()
   }
 } // impl
 
@@ -921,7 +930,7 @@ mod tests {
   fn dp_basic_domain_participant() {
     // let _dp = DomainParticipant::new();
 
-    let sender = UDPSender::new(11401);
+    let sender = UDPSender::new(11401).unwrap();
     let data: Vec<u8> = vec![0, 1, 2, 3, 4];
 
     let addrs = vec![SocketAddr::new("127.0.0.1".parse().unwrap(), 7412)];
@@ -968,7 +977,7 @@ mod tests {
       .expect("Failed to create datawriter");
 
     let portNumber: u16 = get_user_traffic_unicast_port(5, 0);
-    let _sender = UDPSender::new(1234);
+    let sender = UDPSender::new(1234).unwrap();
     let mut m: Message = Message::default();
 
     let a: AckNack = AckNack {
@@ -1012,6 +1021,6 @@ mod tests {
        * 0x00, 0x00, 0x01], */
     };
     let locas = vec![loca];
-    _sender.send_to_locator_list(&_data, &locas);
+    sender.send_to_locator_list(&_data, &locas);
   }
 }
