@@ -18,6 +18,7 @@ use std::{
   collections::BTreeSet,
   iter::FromIterator,
   sync::{Arc, RwLock},
+  rc::Rc,
 };
 use crate::structure::dds_cache::{DDSCache};
 //use std::time::Instant;
@@ -52,11 +53,33 @@ use super::{
 
 use super::qos::InlineQos;
 
+// Some pieces necessary to contruct a reader.
+// These can be sent between threads, whereas a Reader cannot.
+pub (crate) struct ReaderIngredients {
+  pub guid: GUID,
+  pub notification_sender: mio_channel::SyncSender<()>,
+  pub status_sender: mio_channel::SyncSender<DataReaderStatus>,
+  pub topic_name: String,
+  pub qos_policy: QosPolicies,
+  pub data_reader_command_receiver: mio_channel::Receiver<ReaderCommand>, 
+}
+
+impl fmt::Debug for ReaderIngredients {
+  // Need manual implementation, because channels cannot be Dubug formatted.
+  fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
+    f.debug_struct("Reader")
+      .field("my_guid", &self.guid)
+      .field("topic_name", &self.topic_name)
+      .field("qos_policy", &self.qos_policy)
+      .finish()
+  }
+}
 
 pub(crate) struct Reader {
   // Should the instant be sent?
   notification_sender: mio_channel::SyncSender<()>,
   status_sender: mio_channel::SyncSender<DataReaderStatus>,
+  udp_sender: Rc<UDPSender>,
 
   is_stateful: bool, // is this StatefulReader or Statelessreader as per RTPS spec
   // Currently we support only stateful behaviour.
@@ -84,30 +107,28 @@ pub(crate) struct Reader {
 
   timed_event_handler: Option<TimedEventHandler>,
   pub(crate) data_reader_command_receiver: mio_channel::Receiver<ReaderCommand>,
-} // placeholder
+
+} 
 
 impl Reader {
   pub fn new(
-    guid: GUID,
-    notification_sender: mio_channel::SyncSender<()>,
-    status_sender: mio_channel::SyncSender<DataReaderStatus>,
+    i : ReaderIngredients,
     dds_cache: Arc<RwLock<DDSCache>>,
-    topic_name: String,
-    qos_policy: QosPolicies,
-    data_reader_command_receiver: mio_channel::Receiver<ReaderCommand>, //qos_policy: QosPolicies, add later to constructor
+    udp_sender: Rc<UDPSender>,
   ) -> Reader {
-    Reader {
-      notification_sender,
-      status_sender,
+      Reader {
+        notification_sender: i.notification_sender,
+        status_sender: i.status_sender,
+        udp_sender,
+        is_stateful: true, // Do not change this before stateless functionality is implemented.
+        dds_cache,
+        topic_name: i.topic_name,
+        qos_policy: i.qos_policy,
 
-      is_stateful: true,
-      dds_cache,
-      topic_name,
-      qos_policy,
 
       #[cfg(test)]
       seqnum_instant_map: BTreeMap::new(),
-      my_guid: guid ,
+      my_guid: i.guid ,
       enpoint_attributes: EndpointAttributes::default(),
 
       heartbeat_response_delay: StdDuration::new(0, 500_000_000), // 0,5sec
@@ -119,7 +140,7 @@ impl Reader {
       requested_deadline_missed_count: 0,
       offered_incompatible_qos_count: 0,
       timed_event_handler: None,
-      data_reader_command_receiver,
+      data_reader_command_receiver: i.data_reader_command_receiver,
     }
   }
   // TODO: check if it's necessary to implement different handlers for discovery
@@ -309,8 +330,9 @@ impl Reader {
     debug!("update_writer_proxy topic={:?}",self.topic_name);
     match offered_qos.compliance_failure_wrt( &self.qos_policy ) {
       None => { // success, update or insert
+        let writer_id = proxy.remote_writer_guid;
         let count_change =
-          self.matched_writer_update(proxy.clone()); // need clone here to log depending on this
+          self.matched_writer_update(proxy); 
         if count_change > 0 {
           self.writer_match_count_total += count_change;
           self.send_status_change(DataReaderStatus::SubscriptionMatched{
@@ -318,7 +340,7 @@ impl Reader {
               current: CountWithChange::new(self.matched_writers.len() as i32, count_change ),
           });
           info!("Matched new remote writer on topic={:?} writer= {:?}", 
-                self.topic_name, &proxy);
+                self.topic_name, writer_id);
         }
       }
       Some(bad_policy_id) => { // no QoS match
@@ -373,8 +395,7 @@ impl Reader {
     self
       .matched_writers
       .iter()
-      .find(|(&g, _)| g.entityId == entity_id)
-      .is_some()
+      .any(|(&g, _)| g.entityId == entity_id)
   }
 
   #[cfg(test)]
@@ -398,40 +419,77 @@ impl Reader {
     self.matched_writers.get_mut(&remote_writer_guid)
   }
 
+
   // handles regular data message and updates history cache
   pub fn handle_data_msg(&mut self, data: Data, 
       data_flags:BitFlags<DATA_Flags>, mr_state: MessageReceiverState ) 
   {
     //trace!("handle_data_msg entry");
+    let receive_timestamp = Timestamp::now();
+
     let duration = match mr_state.timestamp {
-      Some(ts) => Timestamp::now().duration_since(ts),
+      Some(ts) => receive_timestamp.duration_since(ts),
       None => Duration::DURATION_ZERO,
     };
 
     // checking lifespan for silent dropping of message
-    match self.get_qos().lifespan {
-      Some(ls) => {
-        if ls.duration < duration {
-          return;
-        }
+    if let Some(ls) = self.get_qos().lifespan {
+      if ls.duration < duration {
+        return;
       }
-      None => (),
     }
 
     let writer_guid = GUID::new_with_prefix_and_id(mr_state.source_guid_prefix, data.writer_id);
-    let seq_num = data.writer_sn;
+    let writer_seq_num = data.writer_sn; // for borrow checker
 
-    let instant = Timestamp::now();
+    match Self::data_to_ddsdata(data,data_flags) {
+      Ok(ddsdata) => {
+        self.process_received_data(ddsdata, receive_timestamp, writer_guid, writer_seq_num)    
+      }
+      Err(e) => debug!("Parsing DATA to DDSData failed: {}",e),
+    }
 
-    let mut no_writers = false;
-    trace!("handle_data_msg from {:?} to {:?} no_writers={:?} seq={:?} topic={:?} stateful={:?}", 
-        &writer_guid, data.reader_id, no_writers, seq_num, self.topic_name, self.is_stateful,);
+    
+  }
+
+  pub fn handle_datafrag_msg(&mut self, datafrag: DataFrag, datafrag_flags: BitFlags<DATAFRAG_Flags>,
+     mr_state: MessageReceiverState) 
+  {
+    let writer_guid = GUID::new_with_prefix_and_id(mr_state.source_guid_prefix, datafrag.writer_id);
+    let seq_num = datafrag.writer_sn;
+    let receive_timestamp = Timestamp::now();
+
+    // check if this submessage is expired already
+    if let (Some(source_timestamp), Some(lifespan)) = (mr_state.timestamp, self.get_qos().lifespan) {
+      let elapsed = receive_timestamp.duration_since(source_timestamp);
+      if lifespan.duration < elapsed {
+        info!("DataFrag {:?} from {:?} lifespan exeeded. duration={:?} elapsed={:?}",
+            seq_num, writer_guid, lifespan.duration, elapsed);
+        return
+      }
+    }
+    let writer_seq_num = datafrag.writer_sn; // for borrow checker
+    if let Some(writer_proxy) = self.matched_writer_lookup(writer_guid) {
+      if let Some(complete_ddsdata) = writer_proxy.handle_datafrag(datafrag, datafrag_flags) {
+        self.process_received_data(complete_ddsdata, receive_timestamp, writer_guid, writer_seq_num );  
+      } else {
+        // not yet complete, nothing more to do
+      }
+    } else {
+      info!("Reader got DATAFRAG, but I have no writer proxy")
+    }
+  }
+
+  // common parts of processing DATA or a completed DATAFRAG (when all frags are received)
+  fn process_received_data(&mut self, ddsdata:DDSData, receive_timestamp: Timestamp, writer_guid:GUID, writer_sn: SequenceNumber) {
+    trace!("handle_data_msg from {:?} seq={:?} topic={:?} stateful={:?}", 
+        &writer_guid, writer_sn, self.topic_name, self.is_stateful,);
     if self.is_stateful {
       let my_entityid = self.my_guid.entityId; // to please borrow checker
       if let Some(writer_proxy) = self.matched_writer_lookup(writer_guid) {
-        if writer_proxy.contains_change(seq_num) {
+        if writer_proxy.contains_change(writer_sn) {
           // change already present
-          debug!("handle_data_msg already have this seq={:?}", seq_num);
+          debug!("handle_data_msg already have this seq={:?}", writer_sn);
           if my_entityid == EntityId::ENTITYID_SPDP_BUILTIN_PARTICIPANT_READER {
             debug!("Accepting duplicate message to participant reader.");
             // This is an attmpted workaround to eProsima FastRTPS not
@@ -441,26 +499,81 @@ impl Reader {
           }
         }
         // Add the change and get the instant
-        writer_proxy.received_changes_add(seq_num, instant);
+        writer_proxy.received_changes_add(writer_sn, receive_timestamp);
       } else {
         // no writer proxy found
         info!("handle_data_msg in stateful Reader {:?} has no writer proxy for {:?} topic={:?}",
           my_entityid, writer_guid, self.topic_name, );
-        no_writers = true;
       }
     } else {
       // stateless reader
       todo!()
     }
 
-    self.make_cache_change(data, data_flags, instant, writer_guid, no_writers);
+    self.make_cache_change(ddsdata, receive_timestamp, writer_guid, writer_sn);
+
     // Add to own track-keeping datastructure
     #[cfg(test)]
-    self.seqnum_instant_map.insert(seq_num, instant);
+    self.seqnum_instant_map.insert(writer_sn, receive_timestamp);
 
     self.notify_cache_change();
   }
 
+  fn data_to_ddsdata(data:Data, data_flags:BitFlags<DATA_Flags>) -> Result<DDSData,String> {
+    let representation_identifier = 
+      if data_flags.contains(DATA_Flags::Endianness) { RepresentationIdentifier::CDR_LE } 
+      else { RepresentationIdentifier::CDR_BE };
+
+    match (data.serialized_payload , 
+        data_flags.contains(DATA_Flags::Data) , data_flags.contains(DATA_Flags::Key)) {
+      (Some(sp), true, false ) => { // data
+        Ok(DDSData::new(sp))
+      }
+
+      (Some(sp), false, true  ) => { // key
+        Ok(DDSData::new_disposed_by_key(
+            Self::deduce_change_kind(data.inline_qos, false, representation_identifier), 
+            sp))
+      }
+
+      (None, false, false ) => { // no data, no key. Maybe there is inline QoS?
+        // At least we should find key hash, or we do not know WTF the writer is talking about
+        let key_hash = 
+          match data.inline_qos
+              .as_ref()
+              .map( |iqos| InlineQos::key_hash(iqos).ok() )
+              .flatten().flatten() {
+            Some(h) => Ok(h),
+            None => {
+              warn!("Received DATA that has no payload and no key_hash inline QoS - discarding");
+              Err("DATA with no contents".to_string())
+            }
+          }?;
+        // now, let's try to determine what is the dispose reason
+        let change_kind = 
+          Self::deduce_change_kind(data.inline_qos, false, representation_identifier);
+        Ok(DDSData::new_disposed_by_key_hash(change_kind, key_hash ))
+      }
+
+      (Some(_), true , true  ) => { // payload cannot be both key and data.
+        // RTPS Spec 9.4.5.3.1 Flags in the Submessage Header says
+        // "D=1 and K=1 is an invalid combination in this version of the protocol."
+        warn!("Got DATA that claims to be both data and key - discarding.");
+        Err("Ambiguous data/key received.".to_string())
+      }
+
+      (Some(_), false, false ) => { // data but no data? - this should not be possible
+        warn!("make_cache_change - Flags says no data or key, but got payload!");
+        Err("DATA message has mystery contents".to_string())
+      }
+      (None, true, _ ) | (None, _ , true ) => {
+        warn!("make_cache_change - Where is my SerializedPayload?");
+        Err("DATA message contents missing".to_string())
+      }
+    }
+  }
+
+
   // Returns if responding with ACKNACK?
   // TODO: Return value seems to go unused in callers.
   pub fn handle_heartbeat_msg(
@@ -520,7 +633,7 @@ impl Reader {
       for instant in removed_instances.values() {
         match cache.from_topic_remove_change(&self.topic_name, instant) {
           Some(_) => (),
-          None => warn!("WriterProxy told to remove an instant which was not present"),
+          None => debug!("WriterProxy told to remove an instant which was not present"), // This may be normal?
         }
       }
     }
@@ -552,14 +665,13 @@ impl Reader {
       // We claim to have received all SNs before "base" and produce a set of missing 
       // sequence numbers that are >= base.
       let reader_sn_state =
-        match missing_seqnums.iter().next()  {
+        match missing_seqnums.get(0)  {
           Some(&first_missing) => {
               // Here we assume missing_seqnums are returned in order.
               // Limit the set to maximum that can be sent in acknack submessage..
             SequenceNumberSet
               ::from_base_and_set(first_missing, 
-                &BTreeSet::from_iter(missing_seqnums.iter()
-                                      .map(|s| *s)
+                &BTreeSet::from_iter(missing_seqnums.iter().copied()
                                       .take_while( |sn| sn < &(first_missing + SequenceNumber::from(256)) ))
                 )
             }
@@ -642,8 +754,7 @@ impl Reader {
     //   1. All sequence numbers in the range gapStart <= sequence_number < gapList.base
     let mut removed_changes : BTreeSet<Timestamp> = 
       writer_proxy.irrelevant_changes_range(gap.gap_start, gap.gap_list.base())
-        .values()
-        .map(|s|*s)
+        .values().copied()
         .collect();
 
     //   2. All the sequence numbers that appear explicitly listed in the gapList.
@@ -666,11 +777,6 @@ impl Reader {
     // self.notify_cache_change();
   }
 
-  pub fn handle_datafrag_msg(&mut self, _datafrag: DataFrag, _mr_State: MessageReceiverState) {
-    todo!() // comines frags to data which is handled normally. page 51-53
-            // let data: Data = something..?
-            // self.handle_data_msg(data, mr_state);
-  }
 
   pub fn handle_heartbeatfrag_msg(
     &mut self,
@@ -692,8 +798,8 @@ impl Reader {
         si.change_kind(), // get from inline QoS
         // TODO: What if si.change_kind() gives ALIVE ??
       None => { 
-        if no_writers { ChangeKind::NOT_ALIVE_UNREGISTERED } 
-        else { ChangeKind::NOT_ALIVE_DISPOSED } // TODO: Is this reasonable default?
+        if no_writers { ChangeKind::NotAliveUnregistered } 
+        else { ChangeKind::NotAliveDisposed } // TODO: Is this reasonable default?
       }
     }
   }
@@ -701,73 +807,19 @@ impl Reader {
   // Convert DATA submessage into a CacheChange and update history cache
   fn make_cache_change(
     &mut self,
-    data: Data,
-    data_flags: BitFlags<DATA_Flags>,
-    instant: Timestamp,
+    data: DDSData,
+    receive_timestamp: Timestamp,
     writer_guid: GUID,
-    no_writers: bool,
+    writer_sn: SequenceNumber,
   ) {
-    let representation_identifier = 
-      if data_flags.contains(DATA_Flags::Endianness) { RepresentationIdentifier::CDR_LE } 
-      else { RepresentationIdentifier::CDR_BE };
 
-    let ddsdata = 
-      match (data.serialized_payload , 
-          data_flags.contains(DATA_Flags::Data) , data_flags.contains(DATA_Flags::Key)) {
-        (Some(sp), true, false ) => { // data
-          DDSData::new(sp)
-        }
-
-        (Some(sp), false, true  ) => { // key
-          DDSData::new_disposed_by_key(Self::deduce_change_kind(data.inline_qos, no_writers, representation_identifier), sp)
-        }
-
-        (None, false, false ) => { // no data, no key. Maybe there is inline QoS?
-          // At least we should find key hash, or we do not know WTF the writer is talking about
-          let key_hash = 
-            match data.inline_qos
-                .as_ref()
-                .map( |iqos| InlineQos::key_hash(iqos).ok() )
-                .flatten().flatten() {
-              Some(h) => h,
-              None => {
-                info!("Writer {:?} sent us DATA that has no payload and no key_hash inline QoS - discarding {:?}",
-                  &writer_guid, &data.inline_qos );
-                return
-              }
-            };
-          // now, let's try to determine what is the dispose reason
-          let change_kind = 
-            Self::deduce_change_kind(data.inline_qos, no_writers, representation_identifier);
-          DDSData::new_disposed_by_key_hash(change_kind, key_hash )
-        }
-
-        (Some(_), true , true  ) => { // payload cannot be both key and data.
-          // RTPS Spec 9.4.5.3.1 Flags in the Submessage Header says
-          // "D=1 and K=1 is an invalid combination in this version of the protocol."
-          info!("Writer {:?} sent us DATA that claims to be both data and key - discarding.",
-            writer_guid);
-          return
-        }
-
-        (Some(_), false, false ) => { // data but no data? - this should not be possible
-          error!("make_cache_change - Flags says no data or key, but got payload!");
-          return
-        }
-        (None, true, _ ) | (None, _ , true ) => {
-          error!("make_cache_change - Where is my SerializedPayload?");
-          return
-        }
-
-      };
-
-    let cache_change = CacheChange::new(writer_guid, data.writer_sn, ddsdata);
+    let cache_change = CacheChange::new(writer_guid, writer_sn, data);
     let mut cache = match self.dds_cache.write() {
       Ok(rwlock) => rwlock,
       // TODO: Should we panic here? Are we allowed to continue with poisoned DDSCache?
       Err(e) => panic!("The DDSCache of is poisoned. Error: {}", e),
     };
-    cache.to_topic_add_change(&self.topic_name, &instant, cache_change);
+    cache.to_topic_add_change(&self.topic_name, &receive_timestamp, cache_change);
   }
 
   // notifies DataReaders (or any listeners that history cache has changed for this reader)
@@ -787,9 +839,8 @@ impl Reader {
   }
 
   fn send_acknack(&self, acknack: AckNack, mr_state: MessageReceiverState) {
-    // Should it be saved as an attribute?
-    let sender = UDPSender::new_with_random_port();
-    // TODO: How to determine which flags should be one? Both on atm
+    // Indicate our endianness.
+    // Set final flag to indicate that we are NOT requesting immediate heartbeat response.
     let flags = BitFlags::<ACKNACK_Flags>::from_flag(ACKNACK_Flags::Endianness)
       | BitFlags::<ACKNACK_Flags>::from_flag(ACKNACK_Flags::Final);
 
@@ -822,14 +873,13 @@ impl Reader {
     let bytes = message
       .write_to_vec_with_ctx(Endianness::LittleEndian)
       .unwrap();
-    sender.send_to_locator_list(&bytes, &mr_state.unicast_reply_locator_list);
+    self.udp_sender.send_to_locator_list(&bytes, &mr_state.unicast_reply_locator_list);
   }
 
+  // TODO: This is much duplicate code from send_acknack.
   pub fn send_preemptive_acknacks(&mut self) {
-    let sender = UDPSender::new_with_random_port();
-
-    let flags = BitFlags::<ACKNACK_Flags>::from_flag(ACKNACK_Flags::Endianness)
-      | BitFlags::<ACKNACK_Flags>::from_flag(ACKNACK_Flags::Final);
+    let flags = BitFlags::<ACKNACK_Flags>::from_flag(ACKNACK_Flags::Endianness);
+    // Do not set final flag --> we are requesting immediate heartbeat from writers.
 
     let infodst_flags =
       BitFlags::<INFODESTINATION_Flags>::from_flag(INFODESTINATION_Flags::Endianness);
@@ -872,7 +922,7 @@ impl Reader {
       let bytes = message
         .write_to_vec_with_ctx(Endianness::LittleEndian)
         .unwrap();
-      sender.send_to_locator_list(&bytes, &writer_proxy.unicast_locator_list);
+      self.udp_sender.send_to_locator_list(&bytes, &writer_proxy.unicast_locator_list);
     }
   }
 
@@ -882,12 +932,6 @@ impl Reader {
 } // impl
 
 impl HasQoSPolicy for Reader {
-  // fn set_qos(&mut self, policy: &QosPolicies) -> DDSResult<()> {
-  //   // TODO: check liveliness of qos_policy
-  //   self.qos_policy = policy.clone();
-  //   Ok(())
-  // }
-
   fn get_qos(&self) -> QosPolicies {
     self.qos_policy.clone()
   }
@@ -959,14 +1003,19 @@ mod tests {
       TopicKind::NoKey,
       TypeDesc::new("testi"),
     );
-    let mut reader = Reader::new(
+
+    let reader_ing = ReaderIngredients {
       guid,
-      send,
-      status_sender,
+      notification_sender: send,
+      status_sender: status_sender,
+      topic_name: "test".to_string(),
+      qos_policy: QosPolicies::qos_none(),
+      data_reader_command_receiver: reader_command_receiver,
+    };
+    let mut reader = Reader::new(
+      reader_ing,
       dds_cache,
-      "test".to_string(),
-      QosPolicies::qos_none(),
-      reader_command_receiver,
+      Rc::new(UDPSender::new(0).unwrap()),
     );
 
     let writer_guid = GUID {
@@ -1008,14 +1057,19 @@ mod tests {
       TopicKind::NoKey,
       TypeDesc::new("testi"),
     );
+
+    let reader_ing = ReaderIngredients {
+      guid: new_guid,
+      notification_sender: send,
+      status_sender: status_sender,
+      topic_name: "test".to_string(),
+      qos_policy: QosPolicies::qos_none(),
+      data_reader_command_receiver: reader_command_receiver,
+    };
     let mut new_reader = Reader::new(
-      new_guid,
-      send,
-      status_sender,
+      reader_ing,
       dds_cache.clone(),
-      "test".to_string(),
-      QosPolicies::qos_none(),
-      reader_command_receiver,
+      Rc::new(UDPSender::new(0).unwrap()),
     );
 
     let writer_guid = GUID {
@@ -1067,14 +1121,18 @@ mod tests {
       TopicKind::NoKey,
       TypeDesc::new("testi"),
     );
+    let reader_ing = ReaderIngredients {
+      guid: new_guid,
+      notification_sender: send,
+      status_sender: status_sender,
+      topic_name: "test".to_string(),
+      qos_policy: QosPolicies::qos_none(),
+      data_reader_command_receiver: reader_command_receiver,
+    };
     let mut new_reader = Reader::new(
-      new_guid,
-      send,
-      status_sender,
-      dds_cache,
-      "test".to_string(),
-      QosPolicies::qos_none(),
-      reader_command_receiver,
+      reader_ing,
+      dds_cache.clone(),
+      Rc::new(UDPSender::new(0).unwrap()),
     );
 
     let writer_guid = GUID {
@@ -1198,16 +1256,22 @@ mod tests {
       TopicKind::NoKey,
       TypeDesc::new("testi"),
     );
+
+    let reader_ing = ReaderIngredients {
+      guid: new_guid,
+      notification_sender: send,
+      status_sender: status_sender,
+      topic_name: "test".to_string(),
+      qos_policy: QosPolicies::qos_none(),
+      data_reader_command_receiver: reader_command_receiver,
+    };
     let mut reader = Reader::new(
-      new_guid,
-      send,
-      status_sender,
-      dds_cache,
-      "test".to_string(),
-      QosPolicies::qos_none(),
-      reader_command_receiver,
+      reader_ing,
+      dds_cache.clone(),
+      Rc::new(UDPSender::new(0).unwrap()),
     );
 
+
     let writer_guid = GUID {
       guidPrefix: GuidPrefix::new(&[1; 12]),
       entityId: EntityId::createCustomEntityID([1; 3], EntityKind::WRITER_WITH_KEY_USER_DEFINED),
