@@ -2,7 +2,7 @@ use std::{io};
 use std::sync::{Arc, RwLock};
 use std::marker::PhantomData;
 
-use itertools::Itertools;
+//use itertools::Itertools;
 use serde::de::DeserializeOwned;
 use mio_extras::channel as mio_channel;
 #[allow(unused_imports)]
@@ -160,6 +160,7 @@ where
   <D as Keyed>::K: Key,
   DA: DeserializerAdapter<D>,
 {
+  #[allow(clippy::too_many_arguments)]
   pub(crate) fn new(
     subscriber: Subscriber,
     my_id: EntityId,
@@ -178,7 +179,7 @@ where
         log_and_err_precondition_not_met!("Cannot create new DataReader, DomainParticipant doesn't exist.") ,
     };
 
-    let my_guid = GUID::new_with_prefix_and_id(dp.get_guid_prefix().clone(), my_id);
+    let my_guid = GUID::new_with_prefix_and_id(dp.get_guid_prefix(), my_id);
 
     Ok(Self {
       my_subscriber: subscriber,
@@ -250,14 +251,15 @@ where
     max_samples: usize,
     read_condition: ReadCondition,
   ) -> Result<Vec<DataSample<&D>>> {
+    // Clear notification buffer. This must be done first to avoid race conditions.
+    while self.notification_receiver.try_recv().is_ok() {}
+
     self.fill_local_datasample_cache();
 
     let mut selected = self.datasample_cache.select_keys_for_access(read_condition);
     selected.truncate(max_samples);
 
     let result = self.datasample_cache.read_by_keys(&selected);
-    // clearing receiver buffer
-    while let Ok(_) = self.notification_receiver.try_recv() {}
 
     Ok(result)
   }
@@ -312,6 +314,9 @@ where
     max_samples: usize,
     read_condition: ReadCondition,
   ) -> Result<Vec<DataSample<D>>> {
+    // Clear notification buffer. This must be done first to avoid race conditions.
+    while self.notification_receiver.try_recv().is_ok() {}
+
     self.fill_local_datasample_cache();
     let mut selected = self.datasample_cache.select_keys_for_access(read_condition);
     debug!("take selected count = {}", selected.len() );
@@ -320,9 +325,6 @@ where
     let result = self.datasample_cache.take_by_keys(&selected);
     debug!("take taken count = {}", result.len() );
     
-    // clearing receiver buffer
-    while let Ok(_) = self.notification_receiver.try_recv() {}
-
     Ok(result)
   }
 
@@ -632,26 +634,17 @@ where
     };
 
     let cache_changes = dds_cache.from_topic_get_changes_in_range(
-      &self.my_topic.get_name().to_string(),
+      &self.my_topic.get_name(),
       &self.latest_instant,
       &Timestamp::now(),
     );
 
-    let cache_changes: Vec<(&Timestamp, &CacheChange)> = cache_changes
-      .into_iter()
-      .sorted_by(|(a, _), (b, _)| Ord::cmp(a, b)) // why not sorted already?
-      .collect();
-
-    match cache_changes.last() {
-      Some((last_instant, _)) => self.latest_instant = **last_instant,
-      // No new changes available
-      None => return,
-    };
-
     for ( instant,
           CacheChange { writer_guid, sequence_number: _, data_value }
         ) in cache_changes
     {
+      self.latest_instant = instant; // update our time pointer
+
       match data_value {
         DDSData::DisposeByKey { key: serialized_key , .. } => {
           // TODO: Should be parameterizable by DeserializerAdapter
@@ -661,7 +654,7 @@ where
           {
             Ok(key) => self
               .datasample_cache
-              .add_sample(Err(key), *writer_guid, *instant, None),
+              .add_sample(Err(key), *writer_guid, instant, None),
             Err(e) => {
               warn!("Failed to deserialize key {}, Topic = {}, Type = {:?}", 
                       e, self.my_topic.get_name(), self.my_topic.get_type() );
@@ -676,7 +669,7 @@ where
           match self.datasample_cache.get_key_by_hash(*key_hash) {
             Some(key) => self
               .datasample_cache
-              .add_sample(Err(key), *writer_guid, *instant, None),
+              .add_sample(Err(key), *writer_guid, instant, None),
             /* TODO: How to get source timestamps other then None ?? */
             None => warn!("Tried to dispose with unkonwn key hash: {:x?}", key_hash),
           }
@@ -691,7 +684,7 @@ where
               Ok(payload) => {
                 self
                 .datasample_cache
-                .add_sample(Ok(payload), *writer_guid, *instant, None)
+                .add_sample(Ok(payload), *writer_guid, instant, None)
               }
               Err(e) => {
                 error!("Failed to deserialize bytes: {}, Topic = {}, Type = {:?}", 
@@ -706,6 +699,31 @@ where
               continue // skip this sample, as we cannot decode it                
           }
         }
+        /*
+        DDSData::DataFrags { representation_identifier, bytes_frags } => {
+          // what is our data serialization format (representation identifier) ?
+          if let Some(recognized_rep_id) = 
+              DA::supported_encodings().iter().find(|r| *r == representation_identifier)
+          {
+            match DA::from_vec_bytes(bytes_frags, *recognized_rep_id) {
+              Ok(payload) => {
+                self
+                .datasample_cache
+                .add_sample(Ok(payload), *writer_guid, instant, None)
+              }
+              Err(e) => {
+                error!("Failed to deserialize (DATAFRAG) bytes: {}, Topic = {}, Type = {:?}", 
+                        e, self.my_topic.get_name(), self.my_topic.get_type() );
+                //debug!("Bytes were {:?}",&serialized_payload.value);
+                continue // skip this sample
+              }
+            }
+          } else {
+              warn!("Unknown representation id {:?}.", representation_identifier);
+              //debug!("Serialized payload was {:?}", &serialized_payload);
+              continue // skip this sample, as we cannot decode it                
+          }
+        } */
       }
     }
   }
@@ -724,8 +742,7 @@ where
         .datasample_cache
         .instance_map
         .keys()
-        .next()
-        .map(|k| k.clone()),
+        .next().cloned(),
     }
   }
 
@@ -787,6 +804,9 @@ where
     // Next = select next instance in the order specified by Ord on keys.
     this_or_next: SelectByKey,
   ) -> Result<Vec<DataSample<&D>>> {
+    // Clear notification buffer. This must be done first to avoid race conditions.
+    while self.notification_receiver.try_recv().is_ok() {}
+
     self.fill_local_datasample_cache();
 
     let key = match self.infer_key(instance_key, this_or_next) {
@@ -801,9 +821,6 @@ where
 
     let result = self.datasample_cache.read_by_keys(&selected);
 
-    // clearing receiver buffer
-    while let Ok(_) = self.notification_receiver.try_recv() {}
-
     Ok(result)
   }
 
@@ -860,6 +877,9 @@ where
     // Next = select next instance in the order specified by Ord on keys.
     this_or_next: SelectByKey,
   ) -> Result<Vec<DataSample<D>>> {
+    // Clear notification buffer. This must be done first to avoid race conditions.
+    while self.notification_receiver.try_recv().is_ok() {}
+
     self.fill_local_datasample_cache();
 
     let key = match self.infer_key(instance_key, this_or_next) {
@@ -874,9 +894,6 @@ where
 
     let result = self.datasample_cache.take_by_keys(&selected);
 
-    // clearing receiver buffer
-    while let Ok(_) = self.notification_receiver.try_recv() {}
-
     Ok(result)
   }
 
@@ -1011,10 +1028,10 @@ where
 
   fn change_kind_to_instance_state(c_k: &ChangeKind) -> InstanceState {
     match c_k {
-      ChangeKind::ALIVE => InstanceState::Alive,
-      ChangeKind::NOT_ALIVE_DISPOSED => InstanceState::NotAlive_Disposed,
+      ChangeKind::Alive => InstanceState::Alive,
+      ChangeKind::NotAliveDisposed => InstanceState::NotAlive_Disposed,
       // TODO check this..?
-      ChangeKind::NOT_ALIVE_UNREGISTERED => InstanceState::NotAlive_NoWriters,
+      ChangeKind::NotAliveUnregistered => InstanceState::NotAlive_NoWriters,
     }
   }
   */ 
@@ -1171,8 +1188,10 @@ mod tests {
   use mio_extras::channel as mio_channel;
   use log::info;
   use crate::dds::reader::Reader;
+  use crate::dds::reader::ReaderIngredients;
   use crate::messages::submessages::data::Data;
   use crate::dds::message_receiver::*;
+  use crate::network::udp_sender::UDPSender;
   use crate::structure::guid::{GuidPrefix, EntityKind};
   use crate::structure::sequence_number::SequenceNumber;
   use crate::serialization::{cdr_deserializer::CDRDeserializerAdapter, cdr_serializer::to_bytes};
@@ -1180,11 +1199,8 @@ mod tests {
   use crate::messages::submessages::submessage_elements::serialized_payload::{
     SerializedPayload, RepresentationIdentifier,
   };
-  use std::{
-    thread,
-    time::{self},
-  };
-  use mio::{Events};
+  use std::rc::Rc;
+  //use mio::{Events};
   use crate::messages::submessages::submessage_flag::*;
 
   #[test]
@@ -1207,14 +1223,17 @@ mod tests {
     let reader_id = EntityId::default();
     let reader_guid = GUID::new_with_prefix_and_id(dp.get_guid_prefix(), reader_id);
 
-    let mut new_reader = Reader::new(
-      reader_guid,
-      send,
+    let reader_ing = ReaderIngredients {
+      guid: reader_guid,
+      notification_sender: send,
       status_sender,
-      dp.get_dds_cache(),
-      topic.get_name().to_string(),
-      QosPolicies::qos_none(),
-      reader_command_receiver,
+      topic_name: topic.get_name().to_string(),
+      qos_policy: QosPolicies::qos_none(),
+      data_reader_command_receiver: reader_command_receiver,
+    };
+
+    let mut new_reader = Reader::new(reader_ing, dp.get_dds_cache(), 
+      Rc::new(UDPSender::new_with_random_port().unwrap())
     );
 
     let mut matching_datareader = sub
@@ -1324,15 +1343,16 @@ mod tests {
     let default_id = EntityId::default();
     let reader_guid = GUID::new_with_prefix_and_id(dp.get_guid_prefix(), default_id);
 
-    let mut reader = Reader::new(
-      reader_guid,
-      send,
+    let reader_ing = ReaderIngredients {
+      guid: reader_guid,
+      notification_sender: send,
       status_sender,
-      dp.get_dds_cache(),
-      topic.get_name().to_string(),
-      QosPolicies::qos_none(),
-      reader_command_receiver,
-    );
+      topic_name: topic.get_name().to_string(),
+      qos_policy: QosPolicies::qos_none(),
+      data_reader_command_receiver: reader_command_receiver,
+    };
+
+    let mut reader = Reader::new(reader_ing, dp.get_dds_cache(), Rc::new(UDPSender::new_with_random_port().unwrap()));
 
     let mut datareader = sub
       .create_datareader::<RandomData, CDRDeserializerAdapter<RandomData>>(topic, None)
@@ -1527,6 +1547,7 @@ mod tests {
     assert!(results.unwrap().is_empty());
   }
 
+/* removing this test case, because UDPSender cannot be moved across threads.
   #[test]
   fn dr_wake_up() {
     let dp = DomainParticipant::new(0).expect("Publisher creation failed!");
@@ -1548,15 +1569,16 @@ mod tests {
     let default_id = EntityId::default();
     let reader_guid = GUID::new_with_prefix_and_id(dp.get_guid_prefix(), default_id);
 
-    let mut reader = Reader::new(
-      reader_guid,
-      send,
+    let mut reader_ing = ReaderIngredients {
+      guid: reader_guid,
+      notification_sender: send,
       status_sender,
-      dp.get_dds_cache(),
-      topic.get_name().to_string(),
-      QosPolicies::qos_none(),
-      reader_command_receiver,
-    );
+      topic_name: topic.get_name().to_string(),
+      qos_policy: QosPolicies::qos_none(),
+      data_reader_command_receiver: reader_command_receiver,
+    };
+
+    let reader = Reader::new(reader_ing, dp.get_dds_cache(), Rc::new(UDPSender::new_with_random_port().unwrap()));
 
     let mut datareader = sub
       .create_datareader::<RandomData, CDRDeserializerAdapter<RandomData>>(topic, None)
@@ -1669,4 +1691,5 @@ mod tests {
     handle.join().unwrap();
     assert_eq!(count_to_stop, 3);
   }
+  */
 }
