@@ -47,6 +47,9 @@ use crate::messages::submessages::submessage_elements::serialized_payload::Seria
 use crate::{discovery::data_types::topic_data::SubscriptionBuiltinTopicData, dds::ddsdata::DDSData};
 use super::super::{datasample_cache::DataSampleCache, writer::WriterCommand, };
 
+/// Simplified type for CDR encoding
+pub type DataWriter_CDR<D> = DataWriter<D,CDRSerializerAdapter<D>>;
+
 /// DDS DataWriter for keyed topics
 ///
 /// # Examples
@@ -131,7 +134,7 @@ where
     status_receiver_rec: Receiver<DataWriterStatus>,
   ) -> Result<DataWriter<D, SA>> {
     let entity_id = match guid {
-      Some(g) => g.entityId.clone(),
+      Some(g) => g.entityId,
       None => EntityId::ENTITYID_UNKNOWN,
     };
 
@@ -141,33 +144,30 @@ where
         log_and_err_precondition_not_met!("Cannot create new DataWriter, DomainParticipant doesn't exist.") ,
     };
 
-    let my_guid = GUID::new_with_prefix_and_id(dp.get_guid_prefix().clone(), entity_id );
+    let my_guid = GUID::new_with_prefix_and_id(dp.get_guid_prefix(), entity_id );
 
     match dds_cache.write() {
       Ok(mut cache) => cache.add_new_topic(
-        &String::from(topic.get_name()),
+        &topic.get_name(),
         TopicKind::NoKey,
         topic.get_type(),
       ),
       Err(e) => panic!("DDSCache is poisoned. {:?}", e),
     };
 
-    match topic.get_qos().liveliness {
-      Some(lv) => match lv {
-        Liveliness::Automatic { lease_duration: _ } => (),
-        Liveliness::ManualByParticipant { lease_duration: _ } => {
-          match discovery_command.send(DiscoveryCommand::MANUAL_ASSERT_LIVELINESS) {
-            Ok(_) => (),
-            Err(e) => {
-              error!("Failed to send DiscoveryCommand - Refresh. {:?}", e);
-            }
+    if let Some(lv) = topic.get_qos().liveliness { match lv {
+      Liveliness::Automatic { lease_duration: _ } => (),
+      Liveliness::ManualByParticipant { lease_duration: _ } => {
+        match discovery_command.send(DiscoveryCommand::MANUAL_ASSERT_LIVELINESS) {
+          Ok(_) => (),
+          Err(e) => {
+            error!("Failed to send DiscoveryCommand - Refresh. {:?}", e);
           }
         }
-        Liveliness::ManualByTopic { lease_duration: _ } => (),
-      },
-      None => (),
-    };
-    let qos = topic.get_qos().clone();
+      }
+      Liveliness::ManualByTopic { lease_duration: _ } => (),
+    } };
+    let qos = topic.get_qos();
     Ok(DataWriter {
       my_publisher: publisher,
       my_topic: topic,
@@ -222,24 +222,21 @@ where
 
   // TODO: What is this function? To what part of DDS spec does it correspond to?
   pub fn refresh_manual_liveliness(&self) {
-    match self.get_qos().liveliness {
-      Some(lv) => match lv {
-        Liveliness::Automatic { lease_duration: _ } => (),
-        Liveliness::ManualByParticipant { lease_duration: _ } => {
-          match self
-            .discovery_command
-            .send(DiscoveryCommand::MANUAL_ASSERT_LIVELINESS)
-          {
-            Ok(_) => (),
-            Err(e) => {
-              error!("Failed to send DiscoveryCommand - Refresh. {:?}", e);
-            }
+    if let Some(lv) = self.get_qos().liveliness { match lv {
+      Liveliness::Automatic { lease_duration: _ } => (),
+      Liveliness::ManualByParticipant { lease_duration: _ } => {
+        match self
+          .discovery_command
+          .send(DiscoveryCommand::MANUAL_ASSERT_LIVELINESS)
+        {
+          Ok(_) => (),
+          Err(e) => {
+            error!("Failed to send DiscoveryCommand - Refresh. {:?}", e);
           }
         }
-        Liveliness::ManualByTopic { lease_duration: _ } => (),
-      },
-      None => (),
-    };
+      }
+      Liveliness::ManualByTopic { lease_duration: _ } => (),
+    } };
   }
 
   /// Writes single data instance to a topic.
@@ -833,7 +830,7 @@ where
     let send_buffer = SA::key_to_Bytes( &key  )?; // serialize key
 
     let ddsdata = DDSData::new_disposed_by_key( 
-      ChangeKind::NOT_ALIVE_DISPOSED,
+      ChangeKind::NotAliveDisposed,
       SerializedPayload::new_from_Bytes( SA::output_encoding() , send_buffer) 
     );
     self.cc_upload
