@@ -10,6 +10,7 @@ use mio_extras::channel::{self as mio_channel, SyncSender, TrySendError};
 use mio::Token;
 use std::{
   sync::{RwLock, Arc},
+  rc::Rc,
   collections::{HashSet, BTreeMap, BTreeSet},
   iter::FromIterator,
   cmp::max,
@@ -25,9 +26,11 @@ use crate::structure::duration::Duration;
 use crate::structure::guid::{GuidPrefix, EntityId, GUID};
 use crate::structure::sequence_number::{SequenceNumber};
 use crate::{
-  messages::submessages::submessages::{
-    AckNack,
-  },
+  // messages::submessages::submessages::{
+  //   AckNack,
+  // },
+  messages::submessages::submessages::AckSubmessage,
+
   structure::cache_change::{CacheChange},
   serialization::{Message},
   dds::dp_event_loop::NACK_RESPONSE_DELAY,
@@ -57,6 +60,17 @@ pub enum DeliveryMode {
   Multicast,
 }
 
+// This is used to construct an actual Writer.
+// Ingrediants are sendable between threads, whereas the Writer is not.
+pub(crate) struct WriterIngredients {
+  pub guid: GUID,
+  pub writer_command_receiver: mio_channel::Receiver<WriterCommand>,
+  pub topic_name: String,
+  pub qos_policies: QosPolicies,
+  pub status_sender: SyncSender<DataWriterStatus>,
+}
+
+
 pub(crate) struct Writer {
   pub endianness: Endianness,
   pub heartbeat_message_counter: i32,
@@ -113,7 +127,7 @@ pub(crate) struct Writer {
   requested_incompatible_qos_count: i32, // how many times a Reader requested incompatible QoS
   //message: Option<Message>,
 
-  udp_sender: UDPSender,
+  udp_sender: Rc<UDPSender>,
 
   // This writer can read/write to only one of this DDSCache topic caches identified with my_topic_name
   dds_cache: Arc<RwLock<DDSCache>>,
@@ -168,14 +182,11 @@ struct AckWaiter {
 
 impl Writer {
   pub fn new(
-    guid: GUID,
-    writer_command_receiver: mio_channel::Receiver<WriterCommand>,
+    i: WriterIngredients,
     dds_cache: Arc<RwLock<DDSCache>>,
-    topic_name: String,
-    qos_policies: QosPolicies,
-    status_sender: SyncSender<DataWriterStatus>,
+    udp_sender: Rc<UDPSender>,
   ) -> Writer {
-    let heartbeat_period = match &qos_policies.reliability {
+    let heartbeat_period = match &i.qos_policies.reliability {
       Some(r) => match r {
         Reliability::BestEffort => None,
         Reliability::Reliable {
@@ -186,12 +197,12 @@ impl Writer {
     };
 
     let heartbeat_period = match heartbeat_period {
-      Some(hbp) => match qos_policies.liveliness {
+      Some(hbp) => match i.qos_policies.liveliness {
         Some(lv) => match lv {
           policy::Liveliness::Automatic { lease_duration: _ } => Some(hbp),
           policy::Liveliness::ManualByParticipant { lease_duration: _ } => Some(hbp),
           policy::Liveliness::ManualByTopic { lease_duration } => {
-            let std_dur = Duration::from(lease_duration);
+            let std_dur = lease_duration;
             Some(std_dur / 3)
           }
         },
@@ -210,23 +221,23 @@ impl Writer {
       nack_suppression_duration: Duration::from_millis(0),
       first_change_sequence_number: SequenceNumber::from(1), // first = 1, last = 0
       last_change_sequence_number: SequenceNumber::from(0),  // means we have nothing to write
-      data_max_size_serialized: 999999999,
-      my_guid: guid,
+      data_max_size_serialized: 999999999, // TODO: this is not reasonable
+      my_guid: i.guid,
       //enpoint_attributes: EndpointAttributes::default(),
-      writer_command_receiver,
+      writer_command_receiver: i.writer_command_receiver ,
       readers: BTreeMap::new(),
       matched_readers_count_total: 0,
       requested_incompatible_qos_count: 0,
       endpoint_attributes: EndpointAttributes::default(),
-      udp_sender: UDPSender::new_with_random_port(),
+      udp_sender,
       dds_cache,
-      my_topic_name: topic_name,
+      my_topic_name: i.topic_name,
       sequence_number_to_instant: BTreeMap::new(),
       //key_to_instant: HashMap::new(),
       disposed_sequence_numbers: HashSet::new(),
       timed_event_handler: None,
-      qos_policies,
-      status_sender,
+      qos_policies: i.qos_policies,
+      status_sender: i.status_sender,
       //offered_deadline_status: OfferedDeadlineMissedStatus::new(),
       ack_waiter: None,
     }
@@ -387,7 +398,7 @@ impl Writer {
           let wait_until = self.last_change_sequence_number;
           let readers_pending: BTreeSet<_> = self.readers.iter()
               .filter_map( |(guid,rp)| {
-                  if let Some(_) = rp.qos().reliability() {
+                  if rp.qos().reliability().is_some() {
                     if rp.all_acked_before <= wait_until { Some(*guid) } else { None } // already acked
                   } else { None } // not reliable reader
                 } )
@@ -493,47 +504,45 @@ impl Writer {
 
   /// after heartbeat is handled timer should be set running again.
   fn set_heartbeat_timer(&mut self) {
-    match self.heartbeat_period {
-      Some(period) => {
-        self.timed_event_handler.as_mut().unwrap().set_timeout(
-          &chronoDuration::from(period),
-          TimerMessageType::WriterHeartbeat,
-        );
-        trace!("set heartbeat timer to {:?} in topic {:?}",period, self.topic_name());
-      }
-      None => (),
+    if let Some(period) = self.heartbeat_period {
+      self.timed_event_handler.as_mut().unwrap().set_timeout(
+        &chronoDuration::from(period),
+        TimerMessageType::WriterHeartbeat,
+      );
+      trace!("set heartbeat timer to {:?} in topic {:?}",period, self.topic_name());
     }
   }
 
   /// When receiving an ACKNACK Message indicating a Reader is missing some data samples, the Writer must
   /// respond by either sending the missing data samples, sending a GAP message when the sample is not relevant, or
   /// sending a HEARTBEAT message when the sample is no longer available
-  pub fn handle_ack_nack(&mut self, reader_guid_prefix: GuidPrefix, an: AckNack) {
-    // sanity check
-    if !self.is_reliable() {
-      warn!("Writer {:x?} is best effort! It should not handle acknack messages!", 
-              self.get_entity_id());
-      return
-    }
-    // Update the ReaderProxy
-    let last_seq = self.last_change_sequence_number; // to avoid borrow problems
+  pub fn handle_ack_nack(&mut self, reader_guid_prefix: GuidPrefix, ack_submessage: AckSubmessage) {
+    match ack_submessage {
+      AckSubmessage::AckNack_Variant(ref an) => {
+      // sanity check
+      if !self.is_reliable() {
+        warn!("Writer {:x?} is best effort! It should not handle acknack messages!", 
+                self.get_entity_id());
+        return
+      }
+      // Update the ReaderProxy
+      let last_seq = self.last_change_sequence_number; // to avoid borrow problems
 
-    // sanity check requested sequence numbers
-    match an.reader_sn_state.iter().next().map( i64::from ) {
-      None => (), // ok
-      Some(0) => 
-        warn!("Request for SN zero! : {:?}",an),
-      Some(_) => (), // ok
-    }
+      // sanity check requested sequence numbers
+      match an.reader_sn_state.iter().next().map( i64::from ) {
+        None => (), // ok
+        Some(0) => 
+          warn!("Request for SN zero! : {:?}",an),
+        Some(_) => (), // ok
+      }
     let my_topic = self.my_topic_name.clone(); // for debugging
 
-    self.update_ack_waiters( 
-      GUID::new(reader_guid_prefix, an.reader_id) , 
-      Some(an.reader_sn_state.base()));
+      self.update_ack_waiters( 
+        GUID::new(reader_guid_prefix, an.reader_id) , 
+        Some(an.reader_sn_state.base()));
 
-    if let Some(reader_proxy) = self.lookup_readerproxy_mut(reader_guid_prefix, an.reader_id) {
-
-        reader_proxy.handle_ack_nack(&an, last_seq);
+      if let Some(reader_proxy) = self.lookup_readerproxy_mut(reader_guid_prefix, an.reader_id) {
+        reader_proxy.handle_ack_nack(&ack_submessage, last_seq);
 
         let reader_guid = reader_proxy.remote_reader_guid; // copy to avoid double mut borrow 
         // Sanity Check: if the reader asked for something we did not even advertise yet.
@@ -560,7 +569,7 @@ impl Writer {
         // if we cannot send more data, we are done.
         // This is to prevent empty "repair data" messages from being sent.
         if reader_proxy.all_acked_before > last_seq {
-          return 
+           
         } else {
           // prime timer to send repair data
           reader_proxy.repair_mode = true; // hold sending normal DATA
@@ -569,8 +578,14 @@ impl Writer {
             TimerMessageType::WriterSendRepairData{ to_reader: reader_guid },
           );
         }
+      }
+    }
+    AckSubmessage::NackFrag_Variant(_nackfrag) => {
+      // TODO
+      error!("NACKFRAG Not implemented")
     }
   }
+}
 
   fn update_ack_waiters(&mut self, guid:GUID, acked_before:Option<SequenceNumber>) {
     let mut completed = false;
@@ -653,7 +668,7 @@ impl Writer {
     }
     // Add GAP submessage, if some chache changes could not be found.
     if ! no_longer_relevant.is_empty() {
-      partial_message = partial_message.gap_msg(BTreeSet::from_iter(no_longer_relevant), &self, reader_guid);
+      partial_message = partial_message.gap_msg(BTreeSet::from_iter(no_longer_relevant), self, reader_guid);
     }
     let data_gap_msg = partial_message
       .add_header_and_build(self.my_guid.guidPrefix);
@@ -665,7 +680,7 @@ impl Writer {
       // Try to send repair messages at 5x rate compared to usual deadline rate
       let delay_to_next_message = 
         self.qos_policies.deadline().map( |dl| dl.0 )
-          .unwrap_or(Duration::from_millis(100)) / 5;
+          .unwrap_or_else(|| Duration::from_millis(100)) / 5;
       self.timed_event_handler.as_mut().unwrap().set_timeout(
         &chronoDuration::from(delay_to_next_message),
         TimerMessageType::WriterSendRepairData{ to_reader: reader_proxy.remote_reader_guid },
@@ -687,7 +702,7 @@ impl Writer {
     let acked_by_all_readers = self.readers.values()
           .map(|r| r.acked_up_to_before())
           .min()
-          .unwrap_or(SequenceNumber::zero());
+          .unwrap_or_else(SequenceNumber::zero);
     // If all readers have acked all up to before 5, and depth is 5, we need
     // to keep samples 0..4, i.e. from acked_up_to_before - depth .
     let first_keeper = 
@@ -708,7 +723,7 @@ impl Writer {
   }
 
   fn increase_heartbeat_counter(&mut self) {
-    self.heartbeat_message_counter = self.heartbeat_message_counter + 1;
+    self.heartbeat_message_counter += 1;
   }
 
   fn send_message_to_readers(&self, preferred_mode: DeliveryMode, message: &Message, 
@@ -720,12 +735,14 @@ impl Writer {
     let mut already_sent_to = BTreeSet::new();
 
     macro_rules! send_unless_sent_and_mark {
-      ($loc:expr) => {
-        if already_sent_to.contains($loc) {
-          trace!("Already sent to {:?}", $loc);
-        } else {
-          self.udp_sender.send_to_locator(&buffer, $loc);
-          already_sent_to.insert($loc.clone());
+      ($locs:expr) => {
+        for loc in $locs.iter() {
+          if already_sent_to.contains(loc) {
+            trace!("Already sent to {:?}", loc);
+          } else {
+            self.udp_sender.send_to_locator(&buffer, loc);
+            already_sent_to.insert(loc.clone());
+          }
         }
       }
     }
@@ -734,20 +751,20 @@ impl Writer {
       match ( preferred_mode, 
               reader.unicast_locator_list.iter().find(|l| Locator::isUDP(l) ), 
               reader.multicast_locator_list.iter().find(|l| Locator::isUDP(l) ) ) {
-        (DeliveryMode::Multicast, _ , Some(mc_locator)) => {
-          send_unless_sent_and_mark!(mc_locator);
+        (DeliveryMode::Multicast, _ , Some(_mc_locator)) => {
+          send_unless_sent_and_mark!(reader.multicast_locator_list);
         }
-        (DeliveryMode::Unicast, Some(uc_locator) , _ ) => {
-          send_unless_sent_and_mark!(uc_locator)
+        (DeliveryMode::Unicast, Some(_uc_locator) , _ ) => {
+          send_unless_sent_and_mark!(reader.unicast_locator_list)
         }        
-        (_delivery_mode, _ , Some(mc_locator)) => {
-          send_unless_sent_and_mark!(mc_locator);
+        (_delivery_mode, _ , Some(_mc_locator)) => {
+          send_unless_sent_and_mark!(reader.multicast_locator_list);
         }
-        (_delivery_mode, Some(uc_locator), _ ) => {
-          send_unless_sent_and_mark!(uc_locator)
+        (_delivery_mode, Some(_uc_locator), _ ) => {
+          send_unless_sent_and_mark!(reader.unicast_locator_list)
         }
         (_delivery_mode, None, None ) => {
-          error!("send_message_to_readers: No locators for {:?}",reader);
+          warn!("send_message_to_readers: No locators for {:?}",reader);
         }
       } // match
     }
@@ -783,11 +800,7 @@ impl Writer {
                 current: CountWithChange::new(self.readers.len() as i32 , change)
               });
           // send out hearbeat, so that new reader can catch up
-          match self.qos_policies.reliability {
-            Some(Reliability::Reliable{..}) =>
-              self.notify_new_data_to_all_readers(),
-            _ => (),
-          }
+          if let Some(Reliability::Reliable{..}) = self.qos_policies.reliability { self.notify_new_data_to_all_readers() }
           info!("Matched new remote reader on topic={:?} reader= {:?}", 
                 self.topic_name(), &reader_proxy);
 
@@ -795,7 +808,7 @@ impl Writer {
       }
       Some(bad_policy_id) => {
         // QoS not compliant :(
-        debug!("update_reader_proxy - QoS mismatch {:?}", bad_policy_id);
+        info!("update_reader_proxy - QoS mismatch {:?} topic={:?}", bad_policy_id, self.topic_name());
         self.requested_incompatible_qos_count += 1;
         self.send_status(DataWriterStatus::OfferedIncompatibleQos { 
               count: CountWithChange::new(self.requested_incompatible_qos_count , 1 ),
