@@ -1,7 +1,5 @@
 use serde::{Serialize, Deserialize, de::Error};
 
-use chrono;
-
 use crate::{
   dds::{
     traits::{key::Keyed, Key},
@@ -126,7 +124,7 @@ impl SPDPDiscoveredParticipantData {
     proxy
   }
 
-  pub fn from_participant(
+  pub fn from_local_participant(
     participant: &DomainParticipant,
     lease_duration: Duration,
   ) -> SPDPDiscoveredParticipantData {
@@ -166,7 +164,7 @@ impl SPDPDiscoveredParticipantData {
       default_unicast_locators,
       default_multicast_locators,
       available_builtin_endpoints: BuiltinEndpointSet::from_u32(builtin_endpoints),
-      lease_duration: Some(Duration::from(lease_duration)),
+      lease_duration: Some(lease_duration),
       manual_liveliness_count: 0,
       builtin_endpoint_qos: None,
       entity_name: None,
@@ -192,7 +190,7 @@ impl Serialize for SPDPDiscoveredParticipantData {
   where
     S: serde::Serializer,
   {
-    let builtin_data_serializer = BuiltinDataSerializer::from_participant_data(&self);
+    let builtin_data_serializer = BuiltinDataSerializer::from_participant_data(self);
     builtin_data_serializer.serialize::<S>(serializer, true)
   }
 }
