@@ -78,7 +78,7 @@ impl<'de> Deserialize<'de> for ReaderProxy {
     let res = deserializer.deserialize_any(custom_ds)?;
     res
       .generate_reader_proxy()
-      .ok_or(de::Error::custom("proxy desrialization"))
+      .ok_or_else(|| de::Error::custom("proxy desrialization"))
   }
 }
 
@@ -87,7 +87,7 @@ impl Serialize for ReaderProxy {
   where
     S: serde::Serializer,
   {
-    let builtin_data_serializer = BuiltinDataSerializer::from_reader_proxy(&self);
+    let builtin_data_serializer = BuiltinDataSerializer::from_reader_proxy(self);
     builtin_data_serializer.serialize::<S>(serializer, false)
   }
 }
@@ -222,16 +222,16 @@ impl SubscriptionBuiltinTopicData {
   }
 
   pub fn set_qos(&mut self, qos: &QosPolicies) {
-    self.durability = qos.durability.clone();
-    self.deadline = qos.deadline.clone();
-    self.latency_budget = qos.latency_budget.clone();
-    self.liveliness = qos.liveliness.clone();
-    self.reliability = qos.reliability.clone();
-    self.ownership = qos.ownership.clone();
-    self.destination_order = qos.destination_order.clone();
-    self.time_based_filter = qos.time_based_filter.clone();
-    self.presentation = qos.presentation.clone();
-    self.lifespan = qos.lifespan.clone();
+    self.durability = qos.durability;
+    self.deadline = qos.deadline;
+    self.latency_budget = qos.latency_budget;
+    self.liveliness = qos.liveliness;
+    self.reliability = qos.reliability;
+    self.ownership = qos.ownership;
+    self.destination_order = qos.destination_order;
+    self.time_based_filter = qos.time_based_filter;
+    self.presentation = qos.presentation;
+    self.lifespan = qos.lifespan;
   }
 
   pub fn generate_qos(&self) -> QosPolicies {
@@ -270,7 +270,7 @@ impl Serialize for SubscriptionBuiltinTopicData {
   where
     S: serde::Serializer,
   {
-    let builtin_data_serializer = BuiltinDataSerializer::from_subscription_topic_data(&self);
+    let builtin_data_serializer = BuiltinDataSerializer::from_subscription_topic_data(self);
     builtin_data_serializer.serialize::<S>(serializer, false)
   }
 }
@@ -310,7 +310,7 @@ impl DiscoveredReaderData {
     let reader_proxy = ReaderProxy::new(reader.get_guid());
     let mut subscription_topic_data = SubscriptionBuiltinTopicData::new(
       reader.get_guid(),
-      &topic.get_name().to_string(),
+      &topic.get_name(),
       &topic.get_type().name().to_string(),
       &topic.get_qos(),
     );
@@ -323,7 +323,7 @@ impl DiscoveredReaderData {
     }
   }
 
-  pub fn default(topic_name: &String, type_name: &String) -> DiscoveredReaderData {
+  pub fn default(topic_name: &str, type_name: &str) -> DiscoveredReaderData {
     let rguid = GUID::dummy_test_guid(EntityKind::READER_WITH_KEY_BUILT_IN);
     let reader_proxy = ReaderProxy::new(rguid);
     let subscription_topic_data = SubscriptionBuiltinTopicData::new(
@@ -340,8 +340,8 @@ impl DiscoveredReaderData {
   }
 
   pub(crate) fn update(&mut self, rtps_reader_proxy: &RtpsReaderProxy) {
-    self.reader_proxy.remote_reader_guid = rtps_reader_proxy.remote_reader_guid.clone();
-    self.reader_proxy.expects_inline_qos = rtps_reader_proxy.expects_in_line_qos.clone();
+    self.reader_proxy.remote_reader_guid = rtps_reader_proxy.remote_reader_guid;
+    self.reader_proxy.expects_inline_qos = rtps_reader_proxy.expects_in_line_qos;
     self.reader_proxy.unicast_locator_list = rtps_reader_proxy.unicast_locator_list.clone();
     self.reader_proxy.multicast_locator_list = rtps_reader_proxy.multicast_locator_list.clone();
   }
@@ -378,7 +378,7 @@ impl Serialize for DiscoveredReaderData {
   where
     S: serde::Serializer,
   {
-    let builtin_data_serializer = BuiltinDataSerializer::from_discovered_reader_data(&self);
+    let builtin_data_serializer = BuiltinDataSerializer::from_discovered_reader_data(self);
     builtin_data_serializer.serialize::<S>(serializer, true)
   }
 }
@@ -403,7 +403,7 @@ impl Serialize for DiscoveredReaderData_Key {
   where
     S: serde::Serializer,
   {
-    let builtin_data_serializer = BuiltinDataSerializer::from_discovered_reader_data_key(&self);
+    let builtin_data_serializer = BuiltinDataSerializer::from_discovered_reader_data_key(self);
     builtin_data_serializer.serialize_key::<S>(serializer, true)
   }
 }
@@ -445,7 +445,7 @@ impl<'de> Deserialize<'de> for WriterProxy {
     let res = deserializer.deserialize_any(custom_ds)?;
     res
       .generate_writer_proxy()
-      .ok_or(de::Error::custom("WriterProxy deserialization"))
+      .ok_or_else(|| de::Error::custom("WriterProxy deserialization"))
   }
 }
 
@@ -454,7 +454,7 @@ impl Serialize for WriterProxy {
   where
     S: serde::Serializer,
   {
-    let builtin_data_serializer = BuiltinDataSerializer::from_writer_proxy(&self);
+    let builtin_data_serializer = BuiltinDataSerializer::from_writer_proxy(self);
     builtin_data_serializer.serialize::<S>(serializer, false)
   }
 }
@@ -486,14 +486,14 @@ impl PublicationBuiltinTopicData {
   pub fn new(
     guid: GUID,
     participant_guid: GUID,
-    topic_name: &String,
-    type_name: &String,
+    topic_name: &str,
+    type_name: &str,
   ) -> PublicationBuiltinTopicData {
     PublicationBuiltinTopicData {
       key: guid,
       participant_key: Some(participant_guid),
-      topic_name: topic_name.clone(),
-      type_name: type_name.clone(),
+      topic_name: topic_name.to_string(),
+      type_name: type_name.to_string(),
       durability: None,
       deadline: None,
       latency_budget: None,
@@ -547,7 +547,7 @@ impl<'de> Deserialize<'de> for PublicationBuiltinTopicData {
     let res = deserializer.deserialize_any(custom_ds)?;
     res
       .generate_publication_topic_data()
-      .map_err(|e| de::Error::custom(e))
+      .map_err(de::Error::custom)
   }
 }
 
@@ -556,7 +556,7 @@ impl Serialize for PublicationBuiltinTopicData {
   where
     S: serde::Serializer,
   {
-    let builtin_data_serializer = BuiltinDataSerializer::from_publication_topic_data(&self);
+    let builtin_data_serializer = BuiltinDataSerializer::from_publication_topic_data(self);
     builtin_data_serializer.serialize::<S>(serializer, false)
   }
 }
@@ -616,7 +616,7 @@ impl DiscoveredWriterData {
     let mut publication_topic_data = PublicationBuiltinTopicData::new(
       writer.get_guid(),
       dp.get_guid(),
-      &topic.get_name().to_string(),
+      &topic.get_name(),
       &topic.get_type().name().to_string(),
     );
 
@@ -630,7 +630,7 @@ impl DiscoveredWriterData {
   }
 
   pub(crate) fn update(&mut self, rtps_writer_proxy: &RtpsWriterProxy) {
-    self.writer_proxy.remote_writer_guid = rtps_writer_proxy.remote_writer_guid.clone();
+    self.writer_proxy.remote_writer_guid = rtps_writer_proxy.remote_writer_guid;
     self.writer_proxy.unicast_locator_list = rtps_writer_proxy.unicast_locator_list.clone();
     self.writer_proxy.multicast_locator_list = rtps_writer_proxy.multicast_locator_list.clone();
   }
@@ -645,7 +645,7 @@ impl<'de> Deserialize<'de> for DiscoveredWriterData {
     let res = deserializer.deserialize_any(custom_ds)?;
     res
       .generate_discovered_writer_data()
-      .map_err(|e| de::Error::custom(e))
+      .map_err(de::Error::custom)
   }
 }
 
@@ -654,7 +654,7 @@ impl Serialize for DiscoveredWriterData {
   where
     S: serde::Serializer,
   {
-    let builtin_data_serializer = BuiltinDataSerializer::from_discovered_writer_data(&self);
+    let builtin_data_serializer = BuiltinDataSerializer::from_discovered_writer_data(self);
     builtin_data_serializer.serialize::<S>(serializer, true)
   }
 }
@@ -668,7 +668,7 @@ impl<'de> Deserialize<'de> for DiscoveredWriterData_Key {
     let res = deserializer.deserialize_any(custom_ds)?;
     res
       .generate_discovered_writer_data_key()
-      .map_err(|e| de::Error::custom(e))
+      .map_err(de::Error::custom)
   }
 }
 
@@ -677,7 +677,7 @@ impl Serialize for DiscoveredWriterData_Key {
   where
     S: serde::Serializer,
   {
-    let builtin_data_serializer = BuiltinDataSerializer::from_discovered_writer_data_key(&self);
+    let builtin_data_serializer = BuiltinDataSerializer::from_discovered_writer_data_key(self);
     builtin_data_serializer.serialize_key::<S>(serializer, true)
   }
 }
@@ -731,7 +731,7 @@ impl<'de> Deserialize<'de> for TopicBuiltinTopicData {
   {
     let custom_ds = BuiltinDataDeserializer::new();
     let res = deserializer.deserialize_any(custom_ds)?;
-    res.generate_topic_data().map_err(|e| de::Error::custom(e))
+    res.generate_topic_data().map_err(de::Error::custom)
   }
 }
 
@@ -740,7 +740,7 @@ impl Serialize for TopicBuiltinTopicData {
   where
     S: serde::Serializer,
   {
-    let builtin_data_serializer = BuiltinDataSerializer::from_topic_data(&self);
+    let builtin_data_serializer = BuiltinDataSerializer::from_topic_data(self);
     builtin_data_serializer.serialize::<S>(serializer, false)
   }
 }
@@ -785,7 +785,7 @@ impl<'de> Deserialize<'de> for DiscoveredTopicData {
     let res = deserializer.deserialize_any(custom_ds)?;
     let topic_data = res
       .generate_topic_data()
-      .map_err(|e| de::Error::custom(e))?;
+      .map_err(de::Error::custom)?;
 
     Ok(DiscoveredTopicData::new(topic_data))
   }
@@ -801,6 +801,8 @@ impl Serialize for DiscoveredTopicData {
   }
 }
 
+pub type DiscoveredTopicData_Key = GUID;
+
 impl Keyed for DiscoveredTopicData {
   type K = GUID;
 
