@@ -64,6 +64,18 @@ pub enum DATAFRAG_Flags {
 }
 submessageflag_impls!(DATAFRAG_Flags);
 
+impl DATAFRAG_Flags {
+  pub fn to_DATA_Flags(dff: BitFlags<DATAFRAG_Flags>) -> BitFlags<DATA_Flags> {
+    let mut df : BitFlags<DATA_Flags> = 
+      if dff.contains(DATAFRAG_Flags::Key) { DATA_Flags::Key.into()  }
+      else { DATA_Flags::Data.into() };
+    if dff.contains(DATAFRAG_Flags::Endianness) {  df.insert( DATA_Flags::Endianness) }
+    if dff.contains(DATAFRAG_Flags::InlineQos) { df.insert(DATA_Flags::InlineQos) }
+    if dff.contains(DATAFRAG_Flags::NonStandardPayload) { df.insert(DATA_Flags::NonStandardPayload) }
+    df
+  }
+}
+
 #[derive(BitFlags, Debug, PartialOrd, PartialEq, Ord, Eq, Readable, Clone, Copy)]
 #[repr(u8)]
 pub enum GAP_Flags {
