@@ -1,4 +1,4 @@
-use std::net::{Ipv4Addr, SocketAddr};
+use std::net::{Ipv4Addr, SocketAddr, IpAddr,};
 use std::io;
 
 use mio::Token;
@@ -10,6 +10,8 @@ use socket2::{Socket,Domain, Type, SockAddr, Protocol, };
 
 use bytes::{Bytes,BytesMut};
 
+use crate::network::util::get_local_multicast_ip_addrs;
+
 const MAX_MESSAGE_SIZE : usize = 64 * 1024; // This is max we can get from UDP.
 const MESSAGE_BUFFER_ALLOCATION_CHUNK : usize = 256 * 1024; // must be >= MAX_MESSAGE_SIZE
 
@@ -25,15 +27,10 @@ pub struct UDPListener {
 
 impl Drop for UDPListener {
   fn drop(&mut self) {
-    match self.multicast_group {
-      Some(mcg) => 
-        self
-          .socket
-          .leave_multicast_v4(&mcg, &Ipv4Addr::UNSPECIFIED)
-          .unwrap_or_else(|e| { error!("leave_multicast_group: {:?}",e); } )
-          ,
-      None => (),
-    }
+    if let Some(mcg) = self.multicast_group { self
+    .socket
+    .leave_multicast_v4(&mcg, &Ipv4Addr::UNSPECIFIED)
+    .unwrap_or_else(|e| { error!("leave_multicast_group: {:?}",e); } ) }
   }
 }
 
@@ -43,7 +40,7 @@ impl UDPListener {
   // TODO: Why is is this function even necessary? Doesn't try_bind() do just the same?
 
   fn new_listening_socket(host: &str, port: u16, reuse_addr: bool) -> io::Result<UdpSocket> {
-    let raw_socket = Socket::new(Domain::ipv4(), Type::dgram(), Some(Protocol::udp()) )?;
+    let raw_socket = Socket::new(Domain::IPV4, Type::DGRAM, Some(Protocol::UDP) )?;
 
     // We set ReuseAddr so that other DomainParticipants on this host can
     // bind to the same multicast address and port.
@@ -54,8 +51,11 @@ impl UDPListener {
     }
 
     // MacOS requires this also
-    if reuse_addr {
-      raw_socket.set_reuse_port(true)?; 
+    #[cfg(not(any(target_os = "solaris", target_os = "illumos", windows)))]
+    {
+      if reuse_addr {
+        raw_socket.set_reuse_port(true)?;
+      }
     }
 
     let address = SocketAddr::new(
@@ -63,15 +63,12 @@ impl UDPListener {
         .map_err(|e| io::Error::new(io::ErrorKind::Other, e))?, 
       port);
 
-    match raw_socket.bind( &SockAddr::from(address) ) {
-      Err(e) => {
-        info!("new_socket - cannot bind socket: {:?}",e);
-        return Err(e)
-      }
-      Ok(_) => (), 
+    if let Err(e) = raw_socket.bind( &SockAddr::from(address) ) {
+      info!("new_socket - cannot bind socket: {:?}",e);
+      return Err(e)
     }
 
-    let std_socket = raw_socket.into_udp_socket();
+    let std_socket = std::net::UdpSocket::from( raw_socket );
     std_socket
       .set_nonblocking(true)
       .expect("Failed to set std socket to non blocking.");
@@ -105,7 +102,14 @@ impl UDPListener {
 
     let mio_socket = Self::new_listening_socket(host, port, true)?;
 
-    mio_socket.join_multicast_v4(&multicast_group, &Ipv4Addr::UNSPECIFIED)?;
+    for multicast_if_ipaddr in get_local_multicast_ip_addrs()? {
+      match multicast_if_ipaddr {
+        IpAddr::V4(a) => {
+          mio_socket.join_multicast_v4(&multicast_group, &a)?;
+        }   
+        IpAddr::V6(_a) => error!("UDPListener::new_multicast() not implemented for IpV6") , // TODO
+      }
+    }
 
     Ok(UDPListener { 
       socket: mio_socket, 
@@ -145,7 +149,6 @@ impl UDPListener {
       }
       Err(e) => {
         debug!("UDPListener::get_message failed: {:?}", e);
-        ()
       }
     };
     message
@@ -227,7 +230,7 @@ mod tests {
   #[test]
   fn udpl_single_address() {
     let listener = UDPListener::new_unicast(Token(0), "127.0.0.1", 10001).unwrap();
-    let sender = UDPSender::new_with_random_port();
+    let sender = UDPSender::new_with_random_port().expect("failed to create UDPSender");
 
     let data: Vec<u8> = vec![0, 1, 2, 3, 4];
 
@@ -243,7 +246,7 @@ mod tests {
   #[test]
   fn udpl_multicast_address() {
     let listener = UDPListener::new_multicast(Token(0), "0.0.0.0", 10002,Ipv4Addr::new(239, 255, 0, 1)).unwrap();
-    let sender = UDPSender::new_with_random_port();
+    let sender = UDPSender::new_with_random_port().unwrap();
 
     //setsockopt(sender.socket.as_raw_fd(), IpMulticastLoop, &true)
     //  .expect("Unable set IpMulticastLoop option on socket");
