@@ -154,7 +154,7 @@ impl BuiltinDataDeserializer {
       metatraffic_multicast_locators: self.metatraffic_multicast_locators.clone(),
       default_unicast_locators: self.default_unicast_locators.clone(),
       default_multicast_locators: self.default_multicast_locators.clone(),
-      available_builtin_endpoints: self.available_builtin_endpoints.clone()
+      available_builtin_endpoints: self.available_builtin_endpoints
         .ok_or_else(| | log_and_err_discovery!("available_builtin_endpoints missing"))?,
       lease_duration: self.lease_duration,
       manual_liveliness_count: self.manual_liveliness_count
@@ -276,10 +276,7 @@ impl BuiltinDataDeserializer {
 
     let mut sbtd = SubscriptionBuiltinTopicData::new(key, topic_name, type_name, &qos);
 
-    match self.participant_guid {
-      Some(g) => sbtd.set_participant_key(g),
-      None => (),
-    };
+    if let Some(g) = self.participant_guid { sbtd.set_participant_key(g) };
 
     Ok(sbtd)
   }
@@ -292,9 +289,9 @@ impl BuiltinDataDeserializer {
       key,
       participant_key: self.participant_guid,
       topic_name: self.topic_name.clone()
-        .ok_or(Error::Message("Failed to parse topic name.".to_string()))?,
+        .ok_or_else(|| Error::Message("Failed to parse topic name.".to_string()))?,
       type_name: self.type_name.clone()
-        .ok_or(Error::Message("Failed to parse topic type.".to_string()))?,
+        .ok_or_else(|| Error::Message("Failed to parse topic type.".to_string()))?,
       durability: self.durability,
       deadline: self.deadline,
       latency_budget: self.latency_budget,
@@ -312,9 +309,9 @@ impl BuiltinDataDeserializer {
     Ok(TopicBuiltinTopicData {
       key: self.endpoint_guid,
       name: self.topic_name
-        .ok_or(Error::Message("Failed to parse topic name.".to_string()))?,
+        .ok_or_else(|| Error::Message("Failed to parse topic name.".to_string()))?,
       type_name: self.type_name
-        .ok_or(Error::Message("Failed to parse topic type.".to_string()))?,
+        .ok_or_else(|| Error::Message("Failed to parse topic type.".to_string()))?,
       durability: self.durability,
       deadline: self.deadline,
       latency_budget: self.latency_budget,
@@ -331,7 +328,7 @@ impl BuiltinDataDeserializer {
 
   pub fn generate_discovered_reader_data(self) -> Result<DiscoveredReaderData,Error> {
     let reader_proxy = self.generate_reader_proxy()
-          .ok_or(Error::Message("ReaderProxy deserialization".to_string() ))?;
+          .ok_or_else(|| Error::Message("ReaderProxy deserialization".to_string() ))?;
     let subscription_topic_data = self.generate_subscription_topic_data()?;
     Ok(DiscoveredReaderData {
       reader_proxy,
@@ -342,7 +339,7 @@ impl BuiltinDataDeserializer {
 
   pub fn generate_discovered_writer_data(self) -> Result<DiscoveredWriterData,Error> {
     let writer_proxy = self.generate_writer_proxy()
-          .ok_or(Error::Message("WriterProxy deserialization".to_string() ))?;
+          .ok_or_else(|| Error::Message("WriterProxy deserialization".to_string() ))?;
     let publication_topic_data = self.generate_publication_topic_data()?;
     Ok( DiscoveredWriterData {
       last_updated: Instant::now(),
@@ -376,7 +373,7 @@ impl BuiltinDataDeserializer {
 
   fn parse_data(mut self, buffer: &[u8], rep: RepresentationIdentifier) -> BuiltinDataDeserializer {
     let mut buffer = buffer.to_vec();
-    while self.sentinel.is_none() && buffer.len() > 0 {
+    while self.sentinel.is_none() && !buffer.is_empty() {
       self = self.read_next(&mut buffer, rep);
     }
 
@@ -388,9 +385,9 @@ impl BuiltinDataDeserializer {
     buffer: &mut Vec<u8>,
     rep: RepresentationIdentifier,
   ) -> BuiltinDataDeserializer {
-    let parameter_id = BuiltinDataDeserializer::read_parameter_id(&buffer, rep).unwrap();
+    let parameter_id = BuiltinDataDeserializer::read_parameter_id(buffer, rep).unwrap();
     let mut parameter_length: usize =
-      BuiltinDataDeserializer::read_parameter_length(&buffer, rep).unwrap() as usize;
+      BuiltinDataDeserializer::read_parameter_length(buffer, rep).unwrap() as usize;
 
     if (parameter_length + 4) > buffer.len() {
       parameter_length = buffer.len() - 4;
@@ -400,253 +397,190 @@ impl BuiltinDataDeserializer {
       ParameterId::PID_PARTICIPANT_GUID => {
         let guid: Result<GUID, Error> =
           CDRDeserializerAdapter::from_bytes(&buffer[4..4 + parameter_length], rep);
-        match guid {
-          Ok(gg) => {
-            self.participant_guid = Some(gg);
-            buffer.drain(..4 + parameter_length);
-            return self;
-          }
-          _ => (),
+        if let Ok(gg) = guid {
+          self.participant_guid = Some(gg);
+          buffer.drain(..4 + parameter_length);
+          return self;
         }
       }
       ParameterId::PID_PROTOCOL_VERSION => {
         let version: Result<ProtocolVersion, Error> =
           CDRDeserializerAdapter::from_bytes(&buffer[4..4 + parameter_length], rep);
-        match version {
-          Ok(vv) => {
-            self.protocol_version = Some(vv);
-            buffer.drain(..4 + parameter_length);
-            return self;
-          }
-          _ => (),
+        if let Ok(vv) = version {
+          self.protocol_version = Some(vv);
+          buffer.drain(..4 + parameter_length);
+          return self;
         }
       }
       ParameterId::PID_VENDOR_ID => {
         let vendor: Result<VendorId, Error> =
           CDRDeserializerAdapter::from_bytes(&buffer[4..4 + parameter_length], rep);
-        match vendor {
-          Ok(vv) => {
-            self.vendor_id = Some(vv);
-            buffer.drain(..4 + parameter_length);
-            return self;
-          }
-          _ => (),
+        if let Ok(vv) = vendor {
+          self.vendor_id = Some(vv);
+          buffer.drain(..4 + parameter_length);
+          return self;
         }
       }
       ParameterId::PID_EXPECTS_INLINE_QOS => {
         let inline_qos: Result<bool, Error> =
           CDRDeserializerAdapter::from_bytes(&buffer[4..4 + parameter_length], rep);
-        match inline_qos {
-          Ok(qos) => {
-            self.expects_inline_qos = Some(qos);
-            buffer.drain(..4 + parameter_length);
-            return self;
-          }
-          _ => (),
+        if let Ok(qos) = inline_qos {
+          self.expects_inline_qos = Some(qos);
+          buffer.drain(..4 + parameter_length);
+          return self;
         }
       }
       ParameterId::PID_METATRAFFIC_UNICAST_LOCATOR => {
         let locator: Result<Locator, Error> =
           CDRDeserializerAdapter::from_bytes(&buffer[4..4 + parameter_length], rep);
-        match locator {
-          Ok(loc) => {
-            self.metatraffic_unicast_locators.push(loc);
-            buffer.drain(..4 + parameter_length);
-            return self;
-          }
-          _ => (),
+        if let Ok(loc) = locator {
+          self.metatraffic_unicast_locators.push(loc);
+          buffer.drain(..4 + parameter_length);
+          return self;
         }
       }
       ParameterId::PID_METATRAFFIC_MULTICAST_LOCATOR => {
         let locator: Result<Locator, Error> =
           CDRDeserializerAdapter::from_bytes(&buffer[4..4 + parameter_length], rep);
-        match locator {
-          Ok(loc) => {
-            self.metatraffic_multicast_locators.push(loc);
-            buffer.drain(..4 + parameter_length);
-            return self;
-          }
-          _ => (),
+        if let Ok(loc) = locator {
+          self.metatraffic_multicast_locators.push(loc);
+          buffer.drain(..4 + parameter_length);
+          return self;
         }
       }
       ParameterId::PID_DEFAULT_UNICAST_LOCATOR => {
         let locator: Result<Locator, Error> =
           CDRDeserializerAdapter::from_bytes(&buffer[4..4 + parameter_length], rep);
-        match locator {
-          Ok(loc) => {
-            self.default_unicast_locators.push(loc);
-            buffer.drain(..4 + parameter_length);
-            return self;
-          }
-          _ => (),
+        if let Ok(loc) = locator {
+          self.default_unicast_locators.push(loc);
+          buffer.drain(..4 + parameter_length);
+          return self;
         }
       }
       ParameterId::PID_DEFAULT_MULTICAST_LOCATOR => {
         let locator: Result<Locator, Error> =
           CDRDeserializerAdapter::from_bytes(&buffer[4..4 + parameter_length], rep);
-        match locator {
-          Ok(loc) => {
-            self.default_multicast_locators.push(loc);
-            buffer.drain(..4 + parameter_length);
-            return self;
-          }
-          _ => (),
+        if let Ok(loc) = locator {
+          self.default_multicast_locators.push(loc);
+          buffer.drain(..4 + parameter_length);
+          return self;
         }
       }
       ParameterId::PID_BUILTIN_ENDPOINT_SET => {
         let endpoints: Result<BuiltinEndpointSet, Error> =
           CDRDeserializerAdapter::from_bytes(&buffer[4..4 + parameter_length], rep);
-        match endpoints {
-          Ok(ep) => {
-            self.available_builtin_endpoints = Some(ep);
-            buffer.drain(..4 + parameter_length);
-            return self;
-          }
-          _ => (),
+        if let Ok(ep) = endpoints {
+          self.available_builtin_endpoints = Some(ep);
+          buffer.drain(..4 + parameter_length);
+          return self;
         }
       }
       ParameterId::PID_PARTICIPANT_LEASE_DURATION => {
         let duration: Result<Duration, Error> =
           CDRDeserializerAdapter::from_bytes(&buffer[4..4 + parameter_length], rep);
-        match duration {
-          Ok(dur) => {
-            self.lease_duration = Some(dur);
-            buffer.drain(..4 + parameter_length);
-            return self;
-          }
-          _ => (),
+        if let Ok(dur) = duration {
+          self.lease_duration = Some(dur);
+          buffer.drain(..4 + parameter_length);
+          return self;
         }
       }
       ParameterId::PID_PARTICIPANT_MANUAL_LIVELINESS_COUNT => {
         let count: Result<i32, Error> =
           CDRDeserializerAdapter::from_bytes(&buffer[4..4 + parameter_length], rep);
-        match count {
-          Ok(c) => {
-            self.manual_liveliness_count = Some(c);
-            buffer.drain(..4 + parameter_length);
-            return self;
-          }
-          _ => (),
+        if let Ok(c) = count {
+          self.manual_liveliness_count = Some(c);
+          buffer.drain(..4 + parameter_length);
+          return self;
         }
       }
       ParameterId::PID_BUILTIN_ENDPOINT_QOS => {
         let qos: Result<BuiltinEndpointQos, Error> =
           CDRDeserializerAdapter::from_bytes(&buffer[4..4 + parameter_length], rep);
-        match qos {
-          Ok(q) => {
-            self.builtin_endpoint_qos = Some(q);
-            buffer.drain(..4 + parameter_length);
-            return self;
-          }
-          _ => (),
+        if let Ok(q) = qos {
+          self.builtin_endpoint_qos = Some(q);
+          buffer.drain(..4 + parameter_length);
+          return self;
         }
       }
       ParameterId::PID_ENTITY_NAME => {
         let name: Result<String, Error> =
           CDRDeserializerAdapter::from_bytes(&buffer[4..4 + parameter_length], rep);
-        match name {
-          Ok(n) => {
-            self.entity_name = Some(n);
-            buffer.drain(..4 + parameter_length);
-            return self;
-          }
-          _ => (),
+        if let Ok(n) = name {
+          self.entity_name = Some(n);
+          buffer.drain(..4 + parameter_length);
+          return self;
         }
       }
       ParameterId::PID_ENDPOINT_GUID => {
         let guid: Result<GUID, Error> =
           CDRDeserializerAdapter::from_bytes(&buffer[4..4 + parameter_length], rep);
-        match guid {
-          Ok(gg) => {
-            self.endpoint_guid = Some(gg);
-            buffer.drain(..4 + parameter_length);
-            return self;
-          }
-          _ => (),
+        if let Ok(gg) = guid {
+          self.endpoint_guid = Some(gg);
+          buffer.drain(..4 + parameter_length);
+          return self;
         }
       }
       ParameterId::PID_UNICAST_LOCATOR => {
         let locator: Result<Locator, Error> =
           CDRDeserializerAdapter::from_bytes(&buffer[4..4 + parameter_length], rep);
-        match locator {
-          Ok(loc) => {
-            self.unicast_locator_list.push(loc);
-            buffer.drain(..4 + parameter_length);
-            return self;
-          }
-          _ => (),
+        if let Ok(loc) = locator {
+          self.unicast_locator_list.push(loc);
+          buffer.drain(..4 + parameter_length);
+          return self;
         }
       }
       ParameterId::PID_MULTICAST_LOCATOR => {
         let locator: Result<Locator, Error> =
           CDRDeserializerAdapter::from_bytes(&buffer[4..4 + parameter_length], rep);
-        match locator {
-          Ok(loc) => {
-            self.multicast_locator_list.push(loc);
-            buffer.drain(..4 + parameter_length);
-            return self;
-          }
-          _ => (),
+        if let Ok(loc) = locator {
+          self.multicast_locator_list.push(loc);
+          buffer.drain(..4 + parameter_length);
+          return self;
         }
       }
       ParameterId::PID_TOPIC_NAME => {
         let topic_name: Result<String, Error> =
           CDRDeserializerAdapter::from_bytes(&buffer[4..4 + parameter_length], rep);
-        match topic_name {
-          Ok(name) => {
-            self.topic_name = Some(name);
-            buffer.drain(..4 + parameter_length);
-            return self;
-          }
-          _ => (),
+        if let Ok(name) = topic_name {
+          self.topic_name = Some(name);
+          buffer.drain(..4 + parameter_length);
+          return self;
         }
       }
       ParameterId::PID_TYPE_NAME => {
         let type_name: Result<String, Error> =
           CDRDeserializerAdapter::from_bytes(&buffer[4..4 + parameter_length], rep);
-        match type_name {
-          Ok(name) => {
-            self.type_name = Some(name);
-            buffer.drain(..4 + parameter_length);
-            return self;
-          }
-          _ => (),
+        if let Ok(name) = type_name {
+          self.type_name = Some(name);
+          buffer.drain(..4 + parameter_length);
+          return self;
         }
       }
       ParameterId::PID_DURABILITY => {
         let durability: Result<Durability, Error> =
           CDRDeserializerAdapter::from_bytes(&buffer[4..4 + parameter_length], rep);
-        match durability {
-          Ok(dur) => {
-            self.durability = Some(dur);
-            buffer.drain(..4 + parameter_length);
-            return self;
-          }
-          _ => (),
+        if let Ok(dur) = durability {
+          self.durability = Some(dur);
+          buffer.drain(..4 + parameter_length);
+          return self;
         }
       }
       ParameterId::PID_DEADLINE => {
         let deadline: Result<Deadline, Error> =
           CDRDeserializerAdapter::from_bytes(&buffer[4..4 + parameter_length], rep);
-        match deadline {
-          Ok(dl) => {
-            self.deadline = Some(dl);
-            buffer.drain(..4 + parameter_length);
-            return self;
-          }
-          _ => (),
+        if let Ok(dl) = deadline {
+          self.deadline = Some(dl);
+          buffer.drain(..4 + parameter_length);
+          return self;
         }
       }
       ParameterId::PID_LATENCY_BUDGET => {
         let latency_budget: Result<LatencyBudget, Error> =
           CDRDeserializerAdapter::from_bytes(&buffer[4..4 + parameter_length], rep);
-        match latency_budget {
-          Ok(lb) => {
-            self.latency_budget = Some(lb);
-            buffer.drain(..4 + parameter_length);
-            return self;
-          }
-          _ => (),
+        if let Ok(lb) = latency_budget {
+          self.latency_budget = Some(lb);
+          buffer.drain(..4 + parameter_length);
+          return self;
         }
       }
       ParameterId::PID_LIVELINESS => {
@@ -664,23 +598,20 @@ impl BuiltinDataDeserializer {
         }
         let liveliness: Result<LivelinessData, Error> =
           CDRDeserializerAdapter::from_bytes(&buffer[4..4 + parameter_length], rep);
-        match liveliness {
-          Ok(liv) => {
-            self.liveliness = match liv.kind {
-              LivelinessKind::Automatic => Some(Liveliness::Automatic {
-                lease_duration: liv.lease_duration,
-              }),
-              LivelinessKind::ManualByParticipant => Some(Liveliness::ManualByParticipant {
-                lease_duration: liv.lease_duration,
-              }),
-              LivelinessKind::ManualbyTopic => Some(Liveliness::ManualByTopic {
-                lease_duration: liv.lease_duration,
-              }),
-            };
-            buffer.drain(..4 + parameter_length);
-            return self;
-          }
-          _ => (),
+        if let Ok(liv) = liveliness {
+          self.liveliness = match liv.kind {
+            LivelinessKind::Automatic => Some(Liveliness::Automatic {
+              lease_duration: liv.lease_duration,
+            }),
+            LivelinessKind::ManualByParticipant => Some(Liveliness::ManualByParticipant {
+              lease_duration: liv.lease_duration,
+            }),
+            LivelinessKind::ManualbyTopic => Some(Liveliness::ManualByTopic {
+              lease_duration: liv.lease_duration,
+            }),
+          };
+          buffer.drain(..4 + parameter_length);
+          return self;
         }
       }
       ParameterId::PID_RELIABILITY => {
@@ -692,71 +623,60 @@ impl BuiltinDataDeserializer {
 
         let reliability: Result<ReliabilityBestEffortData, Error> =
           CDRDeserializerAdapter::from_bytes(&buffer[4..4 + parameter_length], rep);
-        match reliability {
-          Ok(rel) => {
-            self.reliability = match rel.reliability_kind {
-              ReliabilityKind::BEST_EFFORT => Some(Reliability::BestEffort),
-              ReliabilityKind::RELIABLE => Some(Reliability::Reliable {
-                max_blocking_time: rel.max_blocking_time,
-              }),
-              _ => {
-                buffer.drain(..4 + parameter_length);
-                return self;
-              }
-            };
+        if let Ok(rel) = reliability {
+          self.reliability = match rel.reliability_kind {
+            ReliabilityKind::BEST_EFFORT => Some(Reliability::BestEffort),
+            ReliabilityKind::RELIABLE => Some(Reliability::Reliable {
+              max_blocking_time: rel.max_blocking_time,
+            }),
+            _ => {
+              buffer.drain(..4 + parameter_length);
+              return self;
+            }
+          };
 
-            buffer.drain(..4 + parameter_length);
-            return self;
-          }
-          _ => (),
+          buffer.drain(..4 + parameter_length);
+          return self;
         }
       }
       ParameterId::PID_OWNERSHIP => {
         #[derive(Deserialize)]
+        #[serde(rename_all = "SCREAMING_SNAKE_CASE")]
         enum OwnershipKind {
-          SHARED,
-          EXCLUSIVE,
+          Shared,
+          Exclusive,
         }
         let ownership: Result<OwnershipKind, Error> =
           CDRDeserializerAdapter::from_bytes(&buffer[4..4 + parameter_length], rep);
-        match ownership {
-          Ok(own) => {
-            let strength = match self.ownership {
-              Some(v) => match v {
-                Ownership::Exclusive { strength } => strength,
-                _ => 0,
-              },
-              None => 0,
-            };
+        if let Ok(own) = ownership {
+          let strength = match self.ownership {
+            Some(Ownership::Exclusive { strength }) => strength,
+            _ => 0,
+          };
 
-            let own = match own {
-              OwnershipKind::SHARED => Ownership::Shared,
-              OwnershipKind::EXCLUSIVE => Ownership::Exclusive { strength: strength },
-            };
+          let own = match own {
+            OwnershipKind::Shared => Ownership::Shared,
+            OwnershipKind::Exclusive => Ownership::Exclusive { strength },
+          };
 
-            self.ownership = Some(own);
-            buffer.drain(..4 + parameter_length);
-            return self;
-          }
-          _ => (),
+          self.ownership = Some(own);
+          buffer.drain(..4 + parameter_length);
+          return self;
         }
       }
       ParameterId::PID_OWNERSHIP_STRENGTH => {
         let ownership_strength: Result<i32, Error> =
           CDRDeserializerAdapter::from_bytes(&buffer[4..4 + parameter_length], rep);
-        match ownership_strength {
-          Ok(stri) => {
-            self.ownership = match self.ownership {
-              Some(v) => match v {
-                Ownership::Exclusive { strength: _ } => {
-                  Some(Ownership::Exclusive { strength: stri })
-                }
-                _ => Some(v),
-              },
-              None => Some(Ownership::Exclusive { strength: stri }),
-            }
+        if let Ok(stri) = ownership_strength {
+          self.ownership = match self.ownership {
+            Some(v) => match v {
+              Ownership::Exclusive { strength: _ } => {
+                Some(Ownership::Exclusive { strength: stri })
+              }
+              _ => Some(v),
+            },
+            None => Some(Ownership::Exclusive { strength: stri }),
           }
-          _ => (),
         };
         buffer.drain(..4 + parameter_length);
         return self;
@@ -764,73 +684,55 @@ impl BuiltinDataDeserializer {
       ParameterId::PID_DESTINATION_ORDER => {
         let destination_order: Result<DestinationOrder, Error> =
           CDRDeserializerAdapter::from_bytes(&buffer[4..4 + parameter_length], rep);
-        match destination_order {
-          Ok(deor) => {
-            self.destination_order = Some(deor);
-            buffer.drain(..4 + parameter_length);
-            return self;
-          }
-          _ => (),
+        if let Ok(deor) = destination_order {
+          self.destination_order = Some(deor);
+          buffer.drain(..4 + parameter_length);
+          return self;
         }
       }
       ParameterId::PID_TIME_BASED_FILTER => {
         let time_based_filter: Result<TimeBasedFilter, Error> =
           CDRDeserializerAdapter::from_bytes(&buffer[4..4 + parameter_length], rep);
-        match time_based_filter {
-          Ok(tbf) => {
-            self.time_based_filter = Some(tbf);
-            buffer.drain(..4 + parameter_length);
-            return self;
-          }
-          _ => (),
+        if let Ok(tbf) = time_based_filter {
+          self.time_based_filter = Some(tbf);
+          buffer.drain(..4 + parameter_length);
+          return self;
         }
       }
       ParameterId::PID_PRESENTATION => {
         let presentation: Result<Presentation, Error> =
           CDRDeserializerAdapter::from_bytes(&buffer[4..4 + parameter_length], rep);
-        match presentation {
-          Ok(p) => {
-            self.presentation = Some(p);
-            buffer.drain(..4 + parameter_length);
-            return self;
-          }
-          _ => (),
+        if let Ok(p) = presentation {
+          self.presentation = Some(p);
+          buffer.drain(..4 + parameter_length);
+          return self;
         }
       }
       ParameterId::PID_LIFESPAN => {
         let lifespan: Result<Lifespan, Error> =
           CDRDeserializerAdapter::from_bytes(&buffer[4..4 + parameter_length], rep);
-        match lifespan {
-          Ok(ls) => {
-            self.lifespan = Some(ls);
-            buffer.drain(..4 + parameter_length);
-            return self;
-          }
-          _ => (),
+        if let Ok(ls) = lifespan {
+          self.lifespan = Some(ls);
+          buffer.drain(..4 + parameter_length);
+          return self;
         }
       }
       ParameterId::PID_CONTENT_FILTER_PROPERTY => {
         let content_filter: Result<ContentFilterProperty, Error> =
           CDRDeserializerAdapter::from_bytes(&buffer[4..4 + parameter_length], rep);
-        match content_filter {
-          Ok(cfp) => {
-            self.content_filter_property = Some(cfp);
-            buffer.drain(..4 + parameter_length);
-            return self;
-          }
-          _ => (),
+        if let Ok(cfp) = content_filter {
+          self.content_filter_property = Some(cfp);
+          buffer.drain(..4 + parameter_length);
+          return self;
         }
       }
       ParameterId::PID_TYPE_MAX_SIZE_SERIALIZED => {
         let max_size: Result<u32, Error> =
           CDRDeserializerAdapter::from_bytes(&buffer[4..4 + parameter_length], rep);
-        match max_size {
-          Ok(ms) => {
-            self.data_max_size_serialized = Some(ms);
-            buffer.drain(..4 + parameter_length);
-            return self;
-          }
-          _ => (),
+        if let Ok(ms) = max_size {
+          self.data_max_size_serialized = Some(ms);
+          buffer.drain(..4 + parameter_length);
+          return self;
         }
       }
       ParameterId::PID_HISTORY => {
@@ -848,29 +750,23 @@ impl BuiltinDataDeserializer {
 
         let history: Result<HistoryData, Error> =
           CDRDeserializerAdapter::from_bytes(&buffer[4..4 + parameter_length], rep);
-        match history {
-          Ok(his) => {
-            let h = match his.kind {
-              HistoryKind::KEEP_LAST => History::KeepLast { depth: his.depth },
-              HistoryKind::KEEP_ALL => History::KeepAll,
-            };
-            self.history = Some(h);
-            buffer.drain(..4 + parameter_length);
-            return self;
-          }
-          _ => (),
+        if let Ok(his) = history {
+          let h = match his.kind {
+            HistoryKind::KEEP_LAST => History::KeepLast { depth: his.depth },
+            HistoryKind::KEEP_ALL => History::KeepAll,
+          };
+          self.history = Some(h);
+          buffer.drain(..4 + parameter_length);
+          return self;
         }
       }
       ParameterId::PID_RESOURCE_LIMITS => {
         let resource_limits: Result<ResourceLimits, Error> =
           CDRDeserializerAdapter::from_bytes(&buffer[4..4 + parameter_length], rep);
-        match resource_limits {
-          Ok(lim) => {
-            self.resource_limits = Some(lim);
-            buffer.drain(..4 + parameter_length);
-            return self;
-          }
-          _ => (),
+        if let Ok(lim) = resource_limits {
+          self.resource_limits = Some(lim);
+          buffer.drain(..4 + parameter_length);
+          return self;
         }
       }
       ParameterId::PID_SENTINEL => {
@@ -885,7 +781,7 @@ impl BuiltinDataDeserializer {
     self
   }
 
-  pub fn read_parameter_id(buffer: &Vec<u8>, rep: RepresentationIdentifier) -> Option<ParameterId> {
+  pub fn read_parameter_id(buffer: &[u8], rep: RepresentationIdentifier) -> Option<ParameterId> {
     let par: Result<ParameterId, Error> = CDRDeserializerAdapter::from_bytes(&buffer[..2], rep);
     match par {
       Ok(val) => Some(val),
@@ -893,7 +789,7 @@ impl BuiltinDataDeserializer {
     }
   }
 
-  pub fn read_parameter_length(buffer: &Vec<u8>, rep: RepresentationIdentifier) -> Option<u16> {
+  pub fn read_parameter_length(buffer: &[u8], rep: RepresentationIdentifier) -> Option<u16> {
     if buffer.len() < 4 {
       return Some(0);
     }
