@@ -44,13 +44,13 @@ struct StringData {
 }
 
 impl StringData {
-  pub fn new(parameter_id: ParameterId, string_data: &String) -> StringData {
+  pub fn new(parameter_id: ParameterId, string_data: &str) -> StringData {
     let parameter_length = string_data.len() as u16;
     let parameter_length = parameter_length + (4 - parameter_length % 4) + 4;
     StringData {
       parameter_id,
       parameter_length,
-      string_data: string_data.clone(),
+      string_data: string_data.to_string(),
     }
   }
 }
@@ -129,7 +129,7 @@ impl BuiltinDataSerializer_Key {
     self.add_participant_guid::<S>(&mut s);
 
     if add_sentinel {
-      s.serialize_field("sentinel", &(1 as u32)).unwrap();
+      s.serialize_field("sentinel", &1_u32).unwrap();
     }
 
     s.end()
@@ -443,7 +443,7 @@ impl<'a> BuiltinDataSerializer<'a> {
       protocol_version: None,
       vendor_id: None,
       expects_inline_qos: None,
-      participant_guid: subscription_topic_data.participant_key().clone(),
+      participant_guid: *subscription_topic_data.participant_key(),
       metatraffic_unicast_locators: None,
       metatraffic_multicast_locators: None,
       default_unicast_locators: None,
@@ -453,22 +453,22 @@ impl<'a> BuiltinDataSerializer<'a> {
       manual_liveliness_count: None,
       builtin_endpoint_qos: None,
       entity_name: None,
-      endpoint_guid: Some(subscription_topic_data.key().clone()),
+      endpoint_guid: Some(subscription_topic_data.key()),
       unicast_locator_list: None,
       multicast_locator_list: None,
       data_max_size_serialized: None,
       topic_name: Some(subscription_topic_data.topic_name()),
       type_name: Some(subscription_topic_data.type_name()),
-      durability: subscription_topic_data.durability().clone(),
-      deadline: subscription_topic_data.deadline().clone(),
-      latency_budget: subscription_topic_data.latency_budget().clone(),
-      liveliness: subscription_topic_data.liveliness().clone(),
-      reliability: subscription_topic_data.reliability().clone(),
-      ownership: subscription_topic_data.ownership().clone(),
-      destination_order: subscription_topic_data.destination_order().clone(),
-      time_based_filter: subscription_topic_data.time_based_filter().clone(),
-      presentation: subscription_topic_data.presentation().clone(),
-      lifespan: subscription_topic_data.lifespan().clone(),
+      durability: *subscription_topic_data.durability(),
+      deadline: *subscription_topic_data.deadline(),
+      latency_budget: *subscription_topic_data.latency_budget(),
+      liveliness: *subscription_topic_data.liveliness(),
+      reliability: *subscription_topic_data.reliability(),
+      ownership: *subscription_topic_data.ownership(),
+      destination_order: *subscription_topic_data.destination_order(),
+      time_based_filter: *subscription_topic_data.time_based_filter(),
+      presentation: *subscription_topic_data.presentation(),
+      lifespan: *subscription_topic_data.lifespan(),
       history: None,
       resource_limits: None,
       content_filter_property: None,
@@ -607,8 +607,7 @@ impl<'a> BuiltinDataSerializer<'a> {
     let bds_ptd = BuiltinDataSerializer::from_publication_topic_data(
       &discovered_writer_data.publication_topic_data,
     );
-    let bds_merged = bds_wp.merge(bds_ptd);
-    bds_merged
+    bds_wp.merge(bds_ptd)
   }
 
   // -----------------------
@@ -673,7 +672,7 @@ impl<'a> BuiltinDataSerializer<'a> {
     self.add_content_filter_property::<S>(&mut s);
 
     if add_sentinel {
-      s.serialize_field("sentinel", &(1 as u32)).unwrap();
+      s.serialize_field("sentinel", &1_u32).unwrap();
     }
 
     s.end()
@@ -692,7 +691,7 @@ impl<'a> BuiltinDataSerializer<'a> {
     self.add_endpoint_guid::<S>(&mut s);
 
     if add_sentinel {
-      s.serialize_field("sentinel", &(1 as u32)).unwrap();
+      s.serialize_field("sentinel", &1_u32).unwrap();
     }
 
     s.end()
@@ -703,46 +702,45 @@ impl<'a> BuiltinDataSerializer<'a> {
     let mut count: usize = 0;
 
     let empty_ll = LocatorList::new();
-    count = count + self.protocol_version.is_some() as usize;
-    count = count + self.vendor_id.is_some() as usize;
-    count = count + self.expects_inline_qos.is_some() as usize;
-    count = count + self.participant_guid.is_some() as usize;
-    count = count + self.metatraffic_unicast_locators.unwrap_or(&empty_ll).len();
-    count = count
-      + self
+    count += self.protocol_version.is_some() as usize;
+    count += self.vendor_id.is_some() as usize;
+    count += self.expects_inline_qos.is_some() as usize;
+    count += self.participant_guid.is_some() as usize;
+    count += self.metatraffic_unicast_locators.unwrap_or(&empty_ll).len();
+    count += self
         .metatraffic_multicast_locators
         .unwrap_or(&empty_ll)
         .len();
-    count = count + self.default_unicast_locators.unwrap_or(&empty_ll).len();
-    count = count + self.default_multicast_locators.unwrap_or(&empty_ll).len();
-    count = count + self.available_builtin_endpoints.is_some() as usize;
-    count = count + self.lease_duration.is_some() as usize;
-    count = count + self.manual_liveliness_count.is_some() as usize;
-    count = count + self.builtin_endpoint_qos.is_some() as usize;
-    count = count + self.entity_name.is_some() as usize;
+    count += self.default_unicast_locators.unwrap_or(&empty_ll).len();
+    count += self.default_multicast_locators.unwrap_or(&empty_ll).len();
+    count += self.available_builtin_endpoints.is_some() as usize;
+    count += self.lease_duration.is_some() as usize;
+    count += self.manual_liveliness_count.is_some() as usize;
+    count += self.builtin_endpoint_qos.is_some() as usize;
+    count += self.entity_name.is_some() as usize;
 
-    count = count + self.endpoint_guid.is_some() as usize;
-    count = count + self.unicast_locator_list.unwrap_or(&empty_ll).len();
-    count = count + self.multicast_locator_list.unwrap_or(&empty_ll).len();
+    count += self.endpoint_guid.is_some() as usize;
+    count += self.unicast_locator_list.unwrap_or(&empty_ll).len();
+    count += self.multicast_locator_list.unwrap_or(&empty_ll).len();
 
-    count = count + self.data_max_size_serialized.is_some() as usize;
+    count += self.data_max_size_serialized.is_some() as usize;
 
-    count = count + self.topic_name.is_some() as usize;
-    count = count + self.type_name.is_some() as usize;
-    count = count + self.durability.is_some() as usize;
-    count = count + self.deadline.is_some() as usize;
-    count = count + self.latency_budget.is_some() as usize;
-    count = count + self.liveliness.is_some() as usize;
-    count = count + self.reliability.is_some() as usize;
-    count = count + self.ownership.is_some() as usize;
-    count = count + self.destination_order.is_some() as usize;
-    count = count + self.time_based_filter.is_some() as usize;
-    count = count + self.presentation.is_some() as usize;
-    count = count + self.lifespan.is_some() as usize;
-    count = count + self.history.is_some() as usize;
-    count = count + self.resource_limits.is_some() as usize;
+    count += self.topic_name.is_some() as usize;
+    count += self.type_name.is_some() as usize;
+    count += self.durability.is_some() as usize;
+    count += self.deadline.is_some() as usize;
+    count += self.latency_budget.is_some() as usize;
+    count += self.liveliness.is_some() as usize;
+    count += self.reliability.is_some() as usize;
+    count += self.ownership.is_some() as usize;
+    count += self.destination_order.is_some() as usize;
+    count += self.time_based_filter.is_some() as usize;
+    count += self.presentation.is_some() as usize;
+    count += self.lifespan.is_some() as usize;
+    count += self.history.is_some() as usize;
+    count += self.resource_limits.is_some() as usize;
 
-    count = count + self.content_filter_property.is_some() as usize;
+    count += self.content_filter_property.is_some() as usize;
 
     count
   }
@@ -775,29 +773,23 @@ impl<'a> BuiltinDataSerializer<'a> {
   }
 
   fn add_expects_inline_qos<S: Serializer>(&self, s: &mut S::SerializeStruct) {
-    match self.expects_inline_qos {
-      Some(iqos) => {
-        let iq = ExpectsInlineQos {
-          parameter_id: ParameterId::PID_EXPECTS_INLINE_QOS,
-          parameter_length: 4,
-          expects_inline_qos: iqos,
-        };
-        s.serialize_field("expects_inline_qos", &iq).unwrap();
-      }
-      None => (),
+    if let Some(iqos) = self.expects_inline_qos {
+      let iq = ExpectsInlineQos {
+        parameter_id: ParameterId::PID_EXPECTS_INLINE_QOS,
+        parameter_length: 4,
+        expects_inline_qos: iqos,
+      };
+      s.serialize_field("expects_inline_qos", &iq).unwrap();
     }
   }
 
   fn add_participant_guid<S: Serializer>(&self, s: &mut S::SerializeStruct) {
-    match self.participant_guid {
-      Some(guid) => {
-        s.serialize_field(
-          "participant_guid",
-          &GUIDData::from(guid, ParameterId::PID_PARTICIPANT_GUID),
-        )
-        .unwrap();
-      }
-      None => (),
+    if let Some(guid) = self.participant_guid {
+      s.serialize_field(
+        "participant_guid",
+        &GUIDData::from(guid, ParameterId::PID_PARTICIPANT_GUID),
+      )
+      .unwrap();
     }
   }
 
@@ -866,77 +858,59 @@ impl<'a> BuiltinDataSerializer<'a> {
   }
 
   fn add_available_builtin_endpoint_set<S: Serializer>(&self, s: &mut S::SerializeStruct) {
-    match self.available_builtin_endpoints {
-      Some(eps) => {
-        s.serialize_field(
-          "available_builtin_endpoints",
-          &BuiltinEndpointSetData::from(eps, ParameterId::PID_BUILTIN_ENDPOINT_SET),
-        )
-        .unwrap();
-      }
-      None => (),
+    if let Some(eps) = self.available_builtin_endpoints {
+      s.serialize_field(
+        "available_builtin_endpoints",
+        &BuiltinEndpointSetData::from(eps, ParameterId::PID_BUILTIN_ENDPOINT_SET),
+      )
+      .unwrap();
     }
   }
 
   fn add_lease_duration<S: Serializer>(&self, s: &mut S::SerializeStruct) {
-    match self.lease_duration {
-      Some(dur) => {
-        s.serialize_field("lease_duration", &DurationData::from(dur))
-          .unwrap();
-      }
-      None => (),
+    if let Some(dur) = self.lease_duration {
+      s.serialize_field("lease_duration", &DurationData::from(dur))
+        .unwrap();
     }
   }
 
   fn add_manual_liveliness_count<S: Serializer>(&self, s: &mut S::SerializeStruct) {
-    match self.manual_liveliness_count {
-      Some(liv) => {
-        let mliv = ManualLivelinessCount {
-          parameter_id: ParameterId::PID_PARTICIPANT_MANUAL_LIVELINESS_COUNT,
-          parameter_length: 4,
-          manual_liveliness_count: liv,
-        };
-        s.serialize_field("manual_liveliness_count", &mliv).unwrap();
-      }
-      None => (),
+    if let Some(liv) = self.manual_liveliness_count {
+      let mliv = ManualLivelinessCount {
+        parameter_id: ParameterId::PID_PARTICIPANT_MANUAL_LIVELINESS_COUNT,
+        parameter_length: 4,
+        manual_liveliness_count: liv,
+      };
+      s.serialize_field("manual_liveliness_count", &mliv).unwrap();
     }
   }
 
   fn add_builtin_endpoint_qos<S: Serializer>(&self, s: &mut S::SerializeStruct) {
-    match self.builtin_endpoint_qos {
-      Some(qos) => {
-        s.serialize_field("builtin_endpoint_qos", &BuiltinEndpointQosData::from(qos))
-          .unwrap();
-      }
-      None => (),
+    if let Some(qos) = self.builtin_endpoint_qos {
+      s.serialize_field("builtin_endpoint_qos", &BuiltinEndpointQosData::from(qos))
+        .unwrap();
     }
   }
 
   fn add_entity_name<S: Serializer>(&self, s: &mut S::SerializeStruct) {
-    match self.entity_name.as_ref() {
-      Some(name) => {
-        let ename = EntityName {
-          parameter_id: ParameterId::PID_ENTITY_NAME,
-          // adding 4 bytes for string lenght and 1 byte for terminate
-          parameter_length: name.len() as u16 + 5,
-          entity_name: name.to_string(),
-        };
-        s.serialize_field("entity_name", &ename).unwrap();
-      }
-      None => (),
+    if let Some(name) = self.entity_name.as_ref() {
+      let ename = EntityName {
+        parameter_id: ParameterId::PID_ENTITY_NAME,
+        // adding 4 bytes for string lenght and 1 byte for terminate
+        parameter_length: name.len() as u16 + 5,
+        entity_name: name.to_string(),
+      };
+      s.serialize_field("entity_name", &ename).unwrap();
     }
   }
 
   fn add_endpoint_guid<S: Serializer>(&self, s: &mut S::SerializeStruct) {
-    match self.endpoint_guid {
-      Some(guid) => {
-        s.serialize_field(
-          "endpoint_guid",
-          &GUIDData::from(guid, ParameterId::PID_ENDPOINT_GUID),
-        )
-        .unwrap();
-      }
-      None => (),
+    if let Some(guid) = self.endpoint_guid {
+      s.serialize_field(
+        "endpoint_guid",
+        &GUIDData::from(guid, ParameterId::PID_ENDPOINT_GUID),
+      )
+      .unwrap();
     }
   }
 
@@ -973,64 +947,49 @@ impl<'a> BuiltinDataSerializer<'a> {
   }
 
   fn add_topic_name<S: Serializer>(&self, s: &mut S::SerializeStruct) {
-    match self.topic_name.as_ref() {
-      Some(name) => {
-        s.serialize_field(
-          "topic_name",
-          &StringData::new(ParameterId::PID_TOPIC_NAME, name),
-        )
-        .unwrap();
-      }
-      None => (),
+    if let Some(name) = self.topic_name.as_ref() {
+      s.serialize_field(
+        "topic_name",
+        &StringData::new(ParameterId::PID_TOPIC_NAME, name),
+      )
+      .unwrap();
     }
   }
 
   fn add_type_name<S: Serializer>(&self, s: &mut S::SerializeStruct) {
-    match self.type_name.as_ref() {
-      Some(name) => {
-        s.serialize_field(
-          "type_name",
-          &StringData::new(ParameterId::PID_TYPE_NAME, name),
-        )
-        .unwrap();
-      }
-      None => (),
+    if let Some(name) = self.type_name.as_ref() {
+      s.serialize_field(
+        "type_name",
+        &StringData::new(ParameterId::PID_TYPE_NAME, name),
+      )
+      .unwrap();
     }
   }
 
   fn add_durability<S: Serializer>(&self, s: &mut S::SerializeStruct) {
-    match self.durability {
-      Some(dur) => {
-        s.serialize_field(
-          "durability",
-          &QosData::new(ParameterId::PID_DURABILITY, dur),
-        )
-        .unwrap();
-      }
-      None => (),
+    if let Some(dur) = self.durability {
+      s.serialize_field(
+        "durability",
+        &QosData::new(ParameterId::PID_DURABILITY, dur),
+      )
+      .unwrap();
     }
   }
 
   fn add_deadline<S: Serializer>(&self, s: &mut S::SerializeStruct) {
-    match self.deadline {
-      Some(dl) => {
-        s.serialize_field("deadline", &QosData::new(ParameterId::PID_DEADLINE, dl))
-          .unwrap();
-      }
-      None => (),
+    if let Some(dl) = self.deadline {
+      s.serialize_field("deadline", &QosData::new(ParameterId::PID_DEADLINE, dl))
+        .unwrap();
     }
   }
 
   fn add_latency_budget<S: Serializer>(&self, s: &mut S::SerializeStruct) {
-    match self.latency_budget {
-      Some(lb) => {
-        s.serialize_field(
-          "latency_budget",
-          &QosData::new(ParameterId::PID_LATENCY_BUDGET, lb),
-        )
-        .unwrap();
-      }
-      None => (),
+    if let Some(lb) = self.latency_budget {
+      s.serialize_field(
+        "latency_budget",
+        &QosData::new(ParameterId::PID_LATENCY_BUDGET, lb),
+      )
+      .unwrap();
     }
   }
 
@@ -1047,34 +1006,31 @@ impl<'a> BuiltinDataSerializer<'a> {
       pub kind: LivelinessKind,
       pub lease_duration: Duration,
     }
-    match self.liveliness {
-      Some(l) => {
-        let data = match l {
-          Liveliness::Automatic { lease_duration } => QosData::new(
-            ParameterId::PID_LIVELINESS,
-            LivelinessData {
-              kind: LivelinessKind::Automatic,
-              lease_duration,
-            },
-          ),
-          Liveliness::ManualByParticipant { lease_duration } => QosData::new(
-            ParameterId::PID_LIVELINESS,
-            LivelinessData {
-              kind: LivelinessKind::ManualByParticipant,
-              lease_duration,
-            },
-          ),
-          Liveliness::ManualByTopic { lease_duration } => QosData::new(
-            ParameterId::PID_LIVELINESS,
-            LivelinessData {
-              kind: LivelinessKind::ManualbyTopic,
-              lease_duration,
-            },
-          ),
-        };
-        s.serialize_field("liveliness", &data).unwrap();
-      }
-      None => (),
+    if let Some(l) = self.liveliness {
+      let data = match l {
+        Liveliness::Automatic { lease_duration } => QosData::new(
+          ParameterId::PID_LIVELINESS,
+          LivelinessData {
+            kind: LivelinessKind::Automatic,
+            lease_duration,
+          },
+        ),
+        Liveliness::ManualByParticipant { lease_duration } => QosData::new(
+          ParameterId::PID_LIVELINESS,
+          LivelinessData {
+            kind: LivelinessKind::ManualByParticipant,
+            lease_duration,
+          },
+        ),
+        Liveliness::ManualByTopic { lease_duration } => QosData::new(
+          ParameterId::PID_LIVELINESS,
+          LivelinessData {
+            kind: LivelinessKind::ManualbyTopic,
+            lease_duration,
+          },
+        ),
+      };
+      s.serialize_field("liveliness", &data).unwrap();
     }
   }
 
@@ -1085,40 +1041,38 @@ impl<'a> BuiltinDataSerializer<'a> {
       pub max_blocking_time: Duration,
     }
 
-    match self.reliability {
-      Some(rel) => match rel {
-        Reliability::BestEffort => {
-          let data = ReliabilityBestEffortData {
-            reliability_kind: ReliabilityKind::BEST_EFFORT,
-            max_blocking_time: Duration::from(StdDuration::from_secs(0)),
-          };
-          s.serialize_field(
-            "reliability",
-            &QosData::new(ParameterId::PID_RELIABILITY, &data),
-          )
-          .unwrap();
-        }
-        Reliability::Reliable { max_blocking_time } => {
-          let data = ReliabilityBestEffortData {
-            reliability_kind: ReliabilityKind::RELIABLE,
-            max_blocking_time: max_blocking_time.clone(),
-          };
-          s.serialize_field(
-            "reliability",
-            &QosData::new(ParameterId::PID_RELIABILITY, &data),
-          )
-          .unwrap();
-        }
-      },
-      None => (),
-    }
+    if let Some(rel) = self.reliability { match rel {
+      Reliability::BestEffort => {
+        let data = ReliabilityBestEffortData {
+          reliability_kind: ReliabilityKind::BEST_EFFORT,
+          max_blocking_time: Duration::from(StdDuration::from_secs(0)),
+        };
+        s.serialize_field(
+          "reliability",
+          &QosData::new(ParameterId::PID_RELIABILITY, &data),
+        )
+        .unwrap();
+      }
+      Reliability::Reliable { max_blocking_time } => {
+        let data = ReliabilityBestEffortData {
+          reliability_kind: ReliabilityKind::RELIABLE,
+          max_blocking_time,
+        };
+        s.serialize_field(
+          "reliability",
+          &QosData::new(ParameterId::PID_RELIABILITY, &data),
+        )
+        .unwrap();
+      }
+    } }
   }
 
   fn add_ownership<S: Serializer>(&self, s: &mut S::SerializeStruct) {
     #[derive(Serialize)]
+    #[serde(rename_all = "SCREAMING_SNAKE_CASE")]
     enum OwnershipKind {
-      SHARED,
-      EXCLUSIVE,
+      Shared,
+      Exclusive,
     }
 
     #[derive(Serialize)]
@@ -1127,86 +1081,71 @@ impl<'a> BuiltinDataSerializer<'a> {
       pub parameter_length: u16,
       pub kind: OwnershipKind,
     }
-    match self.ownership {
-      Some(own) => match own {
-        Ownership::Shared => {
-          s.serialize_field(
-            "ownership",
-            &OwnershipData {
-              parameter_id: ParameterId::PID_OWNERSHIP,
-              parameter_length: 4,
-              kind: OwnershipKind::SHARED,
-            },
-          )
-          .unwrap();
-        }
-        Ownership::Exclusive { strength } => {
-          s.serialize_field(
-            "ownership",
-            &OwnershipData {
-              parameter_id: ParameterId::PID_OWNERSHIP,
-              parameter_length: 4,
-              kind: OwnershipKind::EXCLUSIVE,
-            },
-          )
-          .unwrap();
-          s.serialize_field(
-            "ownership_strength",
-            &I32Data::new(ParameterId::PID_OWNERSHIP_STRENGTH, strength.clone()),
-          )
-          .unwrap();
-        }
-      },
-      None => (),
-    }
-  }
-
-  fn add_destination_order<S: Serializer>(&self, s: &mut S::SerializeStruct) {
-    match self.destination_order {
-      Some(deor) => {
+    if let Some(own) = self.ownership { match own {
+      Ownership::Shared => {
         s.serialize_field(
-          "destination_order",
-          &QosData::new(ParameterId::PID_DESTINATION_ORDER, deor),
+          "ownership",
+          &OwnershipData {
+            parameter_id: ParameterId::PID_OWNERSHIP,
+            parameter_length: 4,
+            kind: OwnershipKind::Shared,
+          },
         )
         .unwrap();
       }
-      None => (),
+      Ownership::Exclusive { strength } => {
+        s.serialize_field(
+          "ownership",
+          &OwnershipData {
+            parameter_id: ParameterId::PID_OWNERSHIP,
+            parameter_length: 4,
+            kind: OwnershipKind::Exclusive,
+          },
+        )
+        .unwrap();
+        s.serialize_field(
+          "ownership_strength",
+          &I32Data::new(ParameterId::PID_OWNERSHIP_STRENGTH, strength),
+        )
+        .unwrap();
+      }
+    } }
+  }
+
+  fn add_destination_order<S: Serializer>(&self, s: &mut S::SerializeStruct) {
+    if let Some(deor) = self.destination_order {
+      s.serialize_field(
+        "destination_order",
+        &QosData::new(ParameterId::PID_DESTINATION_ORDER, deor),
+      )
+      .unwrap();
     }
   }
 
   fn add_time_based_filter<S: Serializer>(&self, s: &mut S::SerializeStruct) {
-    match self.time_based_filter {
-      Some(tbf) => {
-        s.serialize_field(
-          "time_based_filter",
-          &QosData::new(ParameterId::PID_TIME_BASED_FILTER, tbf),
-        )
-        .unwrap();
-      }
-      None => (),
+    if let Some(tbf) = self.time_based_filter {
+      s.serialize_field(
+        "time_based_filter",
+        &QosData::new(ParameterId::PID_TIME_BASED_FILTER, tbf),
+      )
+      .unwrap();
     }
   }
 
   fn add_presentation<S: Serializer>(&self, s: &mut S::SerializeStruct) {
-    match self.presentation {
-      Some(p) => {
-        s.serialize_field(
-          "presentation",
-          &QosData::new(ParameterId::PID_PRESENTATION, p),
-        )
-        .unwrap();
-      }
-      None => (),
+    if let Some(p) = self.presentation {
+      s.serialize_field(
+        "presentation",
+        &QosData::new(ParameterId::PID_PRESENTATION, p),
+      )
+      .unwrap();
     }
   }
 
   fn add_lifespan<S: Serializer>(&self, s: &mut S::SerializeStruct) {
-    match self.lifespan {
-      Some(ls) => {
-        s.serialize_field("lifespan", &QosData::new(ParameterId::PID_LIFESPAN, ls))
-          .unwrap();
-      }
-      None => (),
+    if let Some(ls) = self.lifespan {
+      s.serialize_field("lifespan", &QosData::new(ParameterId::PID_LIFESPAN, ls))
+        .unwrap();
     }
   }
 
@@ -1223,64 +1162,52 @@ impl<'a> BuiltinDataSerializer<'a> {
       pub depth: i32,
     }
 
-    match self.history {
-      Some(hs) => {
-        let history_data = match hs {
-          History::KeepLast { depth } => HistoryData {
-            kind: HistoryKind::KEEP_LAST,
-            depth: depth,
-          },
-          History::KeepAll => HistoryData {
-            kind: HistoryKind::KEEP_ALL,
-            depth: 0,
-          },
-        };
-        s.serialize_field(
-          "history",
-          &QosData::new(ParameterId::PID_HISTORY, &history_data),
-        )
-        .unwrap();
-      }
-      None => (),
+    if let Some(hs) = self.history {
+      let history_data = match hs {
+        History::KeepLast { depth } => HistoryData {
+          kind: HistoryKind::KEEP_LAST,
+          depth,
+        },
+        History::KeepAll => HistoryData {
+          kind: HistoryKind::KEEP_ALL,
+          depth: 0,
+        },
+      };
+      s.serialize_field(
+        "history",
+        &QosData::new(ParameterId::PID_HISTORY, &history_data),
+      )
+      .unwrap();
     }
   }
 
   fn add_resource_limits<S: Serializer>(&self, s: &mut S::SerializeStruct) {
-    match self.resource_limits {
-      Some(rl) => {
-        s.serialize_field(
-          "resource_limits",
-          &QosData::new(ParameterId::PID_RESOURCE_LIMITS, rl),
-        )
-        .unwrap();
-      }
-      None => (),
+    if let Some(rl) = self.resource_limits {
+      s.serialize_field(
+        "resource_limits",
+        &QosData::new(ParameterId::PID_RESOURCE_LIMITS, rl),
+      )
+      .unwrap();
     }
   }
 
   fn add_content_filter_property<S: Serializer>(&self, s: &mut S::SerializeStruct) {
-    match self.content_filter_property {
-      Some(cfp) => {
-        s.serialize_field(
-          "content_filter_property",
-          &ContentFilterPropertyData::new(cfp),
-        )
-        .unwrap();
-      }
-      None => (),
+    if let Some(cfp) = self.content_filter_property {
+      s.serialize_field(
+        "content_filter_property",
+        &ContentFilterPropertyData::new(cfp),
+      )
+      .unwrap();
     }
   }
 
   fn add_data_max_size_serialized<S: Serializer>(&self, s: &mut S::SerializeStruct) {
-    match self.data_max_size_serialized {
-      Some(dmss) => {
-        s.serialize_field(
-          "data_max_size_serialized",
-          &U32Data::new(ParameterId::PID_TYPE_MAX_SIZE_SERIALIZED, dmss),
-        )
-        .unwrap();
-      }
-      None => (),
+    if let Some(dmss) = self.data_max_size_serialized {
+      s.serialize_field(
+        "data_max_size_serialized",
+        &U32Data::new(ParameterId::PID_TYPE_MAX_SIZE_SERIALIZED, dmss),
+      )
+      .unwrap();
     }
   }
 }
