@@ -86,7 +86,7 @@ impl<'a> Message {
     let mut message = Message::new(rtps_header);
     let mut submessages_left : Bytes = buffer.slice(20..); // header is 20 bytes
                                               // submessage loop
-    while submessages_left.len() > 0 {
+    while !submessages_left.is_empty() {
       let sub_header = SubmessageHeader::read_from_buffer(&submessages_left)
         .map_err(|e| io::Error::new(io::ErrorKind::Other, e))?;
       // Try to figure out how large this submessage is.
@@ -258,7 +258,7 @@ impl Default for Message {
 }
 
 impl<C: Context> Writable<C> for Message {
-  fn write_to<'a, T: ?Sized + Writer<C>>(&'a self, writer: &mut T) -> Result<(), C::Error> {
+  fn write_to<T: ?Sized + Writer<C>>(&self, writer: &mut T) -> Result<(), C::Error> {
     writer.write_value(&self.header)?;
     for x in &self.submessages {
       writer.write_value(&x)?;
@@ -342,29 +342,9 @@ impl MessageBuilder {
     writer_entity_id: EntityId,
     endianness: Endianness,
   ) -> MessageBuilder {
-    //use std::convert::TryFrom;
     let inline_qos = match cache_change.data_value {
-      DDSData::Data {..} => None,
-      DDSData::DisposeByKey { /*ref key,*/ .. } => {
-        /*let mut param_list = ParameterList::new();
-        let mut digest_bytes : Vec<u8> = 
-          if key.value.len() > 16 {
-            md5::compute(&key.value).to_vec()
-          } else {
-            key.value.to_vec()
-          };
-        digest_bytes.resize(16,0x00); // append zeroes to 16 bytes
-        let key_hash = u128::from_le_bytes(<[u8;16]>::try_from(digest_bytes).unwrap());
-        param_list.parameters.push(
-          Parameter {
-            parameter_id: ParameterId::PID_KEY_HASH,
-            value: key_hash.to_le_bytes().to_vec(),
-          });
-        param_list.parameters.push( 
-          Parameter::create_pid_status_info_parameter(true, true, false) );
-        Some( param_list ) */
-        None
-      },
+      DDSData::Data {..} /*| DDSData::DataFrags{..}*/  => None,
+      DDSData::DisposeByKey {..} => None,
       DDSData::DisposeByKeyHash{ key_hash, .. } => {
         let mut param_list = ParameterList::new();
         let key_hash_param = Parameter {
@@ -394,16 +374,14 @@ impl MessageBuilder {
     
     // TODO: please explain this logic here:
     if writer_entity_id.kind() == EntityKind::WRITER_WITH_KEY_BUILT_IN {
-      match data_message.serialized_payload.as_mut() {
-        Some(sp) => sp.representation_identifier = RepresentationIdentifier::PL_CDR_LE,
-        None => (),
-      }
+      if let Some(sp) = data_message.serialized_payload.as_mut() { sp.representation_identifier = RepresentationIdentifier::PL_CDR_LE }
     }
 
     let flags: BitFlags<DATA_Flags> = 
       BitFlags::<DATA_Flags>::from_endianness(endianness)
       | ( match cache_change.data_value {
-           DDSData::Data {..} => BitFlags::<DATA_Flags>::from_flag(DATA_Flags::Data),
+           DDSData::Data {..} /*| DDSData::DataFrags {..} */ 
+              => BitFlags::<DATA_Flags>::from_flag(DATA_Flags::Data),
            DDSData::DisposeByKey{..} => BitFlags::<DATA_Flags>::from_flag(DATA_Flags::Key),
            DDSData::DisposeByKeyHash{..} => BitFlags::<DATA_Flags>::from_flag(DATA_Flags::InlineQos),
            //_ => BitFlags::<DATA_Flags>::from_flag(DATA_Flags::InlineQos),
