@@ -1,10 +1,11 @@
 use crate::dds::data_types::GUID;
 use std::collections::BTreeSet;
 use crate::structure::sequence_number::SequenceNumber;
-use log::{error, trace};
+
+#[allow(unused_imports)] use log::{debug, error, info,trace};
 
 use std::{
-  collections::{BTreeMap, HashMap, btree_map::Range},
+  collections::{BTreeMap, HashMap, /*btree_map::Range*/ },
   cmp::max,
 };
 
@@ -39,121 +40,88 @@ impl DDSCache {
 
   pub fn add_new_topic(
     &mut self,
-    topic_name: &String,
+    topic_name: &str,
     topic_kind: TopicKind,
     topic_data_type: TypeDesc,
-  ) -> bool {
-    if self.topic_caches.contains_key(topic_name) {
-      false
-    } else {
-      self.topic_caches.insert(
+  ) {
+    self.topic_caches.insert(
         topic_name.to_string(),
-        TopicCache::new(topic_kind, topic_data_type),
+        TopicCache::new(topic_name, topic_kind, topic_data_type),
       );
-      true
-    }
   }
 
-  pub fn remove_topic(&mut self, topic_name: &String) {
+  pub fn remove_topic(&mut self, topic_name: &str) {
     if self.topic_caches.contains_key(topic_name) {
       self.topic_caches.remove(topic_name);
     }
   }
 
-  pub fn get_topic_qos_mut(&mut self, topic_name: &String) -> Option<&mut QosPolicies> {
-    if self.topic_caches.contains_key(topic_name) {
-      Some(&mut self.topic_caches.get_mut(topic_name).unwrap().topic_qos)
-    } else {
-      None
-    }
-  }
-
-  pub fn get_topic_qos(&self, topic_name: &String) -> Option<&QosPolicies> {
-    if self.topic_caches.contains_key(topic_name) {
-      Some(&self.topic_caches.get(topic_name).unwrap().topic_qos)
-    } else {
-      None
-    }
-  }
-
-  pub fn from_topic_get_change(&self, topic_name: &String, instant: &Timestamp) 
+  pub fn from_topic_get_change(&self, topic_name: &str, instant: &Timestamp) 
     -> Option<&CacheChange> 
   {
-    self.topic_caches.get(topic_name).map( |tc| tc.get_change(instant) ).flatten()
+    self.topic_caches.get(topic_name)
+      .map( |tc| tc.get_change(instant) )
+      .flatten()
   }
 
 
   /// Removes cacheChange permanently
   pub fn from_topic_remove_change(
     &mut self,
-    topic_name: &String,
+    topic_name: &str,
     instant: &Timestamp,
   ) -> Option<CacheChange> {
     match self.topic_caches.get_mut(topic_name) {
       Some(tc) => tc.remove_change(instant),
       None => {
-        error!("Topic: '{:?}' is not in DDSCache", topic_name); 
+        error!("from_topic_remove_change: Topic {:?} is not in DDSCache", topic_name); 
         None  
       }
     }
   }
 
   /// Removes cacheChange permanently
-  pub fn from_topic_remove_before(&mut self, topic_name: &String, instant: Timestamp) 
+  pub fn from_topic_remove_before(&mut self, topic_name: &str, instant: Timestamp) 
   {
     match self.topic_caches.get_mut(topic_name) {
       Some(tc) => tc.remove_changes_before(instant),
       None => {
-        error!("Topic: '{:?}' is not in DDSCache", topic_name); 
+        error!("from_topic_remove_before: topic: {:?} is not in DDSCache", topic_name); 
       }
     }
   }
 
 
-  pub fn from_topic_get_all_changes(&self, topic_name: &str) -> Vec<(&Timestamp, &CacheChange)> {
-    match self.topic_caches.get(topic_name) {
-      Some(r) => r.get_all_changes(),
-      None => vec![],
-    }
-  }
-
   pub fn from_topic_get_changes_in_range(
     &self,
-    topic_name: &String,
+    topic_name: &str,
     start_instant: &Timestamp,
     end_instant: &Timestamp,
-  ) -> Vec<(&Timestamp, &CacheChange)> {
-    if self.topic_caches.contains_key(topic_name) {
-      return self
-        .topic_caches
-        .get(topic_name)
-        .unwrap()
-        .get_changes_in_range(start_instant, end_instant);
-    } else {
-      return vec![];
+  ) -> Box<dyn Iterator<Item=(Timestamp, &CacheChange)> + '_> {
+    match self.topic_caches.get(topic_name) {
+      Some(tc) => Box::new(tc.get_changes_in_range(start_instant, end_instant)),
+      None => Box::new(vec![].into_iter()),
     }
   }
 
   pub fn to_topic_add_change(
     &mut self,
-    topic_name: &String,
+    topic_name: &str,
     instant: &Timestamp,
     cache_change: CacheChange,
   ) {
-    if self.topic_caches.contains_key(topic_name) {
-      return self
-        .topic_caches
-        .get_mut(topic_name)
-        .unwrap()
-        .add_change(instant, cache_change);
-    } else {
-      error!("Topic: '{:?}' is not added to DDSCache", topic_name);
+    match self.topic_caches.get_mut(topic_name) {
+      Some(tc) => tc.add_change(instant, cache_change),
+      None => {
+        error!("to_topic_add_change: Topic: {:?} is not in DDSCache", topic_name); 
+      }
     }
   }
 }
 
 #[derive(Debug)]
 pub struct TopicCache {
+  topic_name: String,
   topic_data_type: TypeDesc,
   topic_kind: TopicKind,
   topic_qos: QosPolicies,
@@ -161,10 +129,11 @@ pub struct TopicCache {
 }
 
 impl TopicCache {
-  pub fn new(topic_kind: TopicKind, topic_data_type: TypeDesc) -> TopicCache {
+  pub fn new(topic_name: &str, topic_kind: TopicKind, topic_data_type: TypeDesc) -> TopicCache {
     TopicCache {
-      topic_data_type: topic_data_type,
-      topic_kind: topic_kind,
+      topic_name: topic_name.to_string(),
+      topic_data_type,
+      topic_kind,
       topic_qos: QosPolicyBuilder::new().build(),
       history_cache: DDSHistoryCache::new(),
     }
@@ -176,20 +145,21 @@ impl TopicCache {
 
   pub fn add_change(&mut self, instant: &Timestamp, cache_change: CacheChange) {
     self.history_cache.add_change(instant, cache_change)
-  }
-
-  pub fn get_all_changes(&self) -> Vec<(&Timestamp, &CacheChange)> {
-    self.history_cache.get_all_changes()
+      .map( |cc_back| { 
+        debug!("DDSCache insert failed topic={:?} cache_change={:?}",
+                self.topic_name, cc_back); 
+            }
+          );
   }
 
   pub fn get_changes_in_range(
     &self,
     start_instant: &Timestamp,
     end_instant: &Timestamp,
-  ) -> Vec<(&Timestamp, &CacheChange)> {
+  ) -> Box<dyn Iterator<Item=(Timestamp, &CacheChange)> + '_> {
     self
       .history_cache
-      .get_range_of_changes_vec(start_instant, end_instant)
+      .get_range_of_changes(start_instant, end_instant)
   }
 
   ///Removes and returns value if it was found
@@ -257,29 +227,26 @@ impl DDSHistoryCache {
     //TODO: If this makes a SN set empty, remove it from BTreeMap.
   }
 
-  pub fn add_change(&mut self, instant: &Timestamp, cache_change: CacheChange) {
+  pub fn add_change(&mut self, instant: &Timestamp, cache_change: CacheChange) -> Option<CacheChange> {
     if self.have_sn(&cache_change) {
       trace!("Received duplicate {:?} from {:?}, discarding.",
         cache_change.sequence_number, cache_change.writer_guid);
-      // nothing else
+      Some(cache_change)
     } else {
       self.insert_sn(&cache_change);
       let result = self.changes.insert(*instant, cache_change);
       match result {
-        None => (), // all is good. timestamp was not inserted before.
+        None => None, // all is good. timestamp was not inserted before.
         Some(old_cc) => {
           // If this happens cahce changes were created at exactly same instant.
-          error!("DDSHistoryCache already contained element with key {:?} !!!", instant);
-          self.remove_sn(&old_cc)
+          error!("DDSHistoryCache already contained element with key {:?} !!!", instant);
+          self.remove_sn(&old_cc);
+          Some(old_cc)
         }
       }
     }
   }
 
-  pub fn get_all_changes(&self) -> Vec<(&Timestamp, &CacheChange)> {
-    self.changes.iter().collect()
-  }
-
   pub fn get_change(&self, instant: &Timestamp) -> Option<&CacheChange> {
     self.changes.get(instant)
   }
@@ -288,25 +255,12 @@ impl DDSHistoryCache {
     &self,
     start_instant: &Timestamp,
     end_instant: &Timestamp,
-  ) -> Range<Timestamp, CacheChange> {
-    self
-      .changes
-      .range((Included(start_instant), Included(end_instant)))
-  }
-
-  pub fn get_range_of_changes_vec(
-    &self,
-    start_instant: &Timestamp,
-    end_instant: &Timestamp,
-  ) -> Vec<(&Timestamp, &CacheChange)> {
-    let mut changes: Vec<(&Timestamp, &CacheChange)> = vec![];
-    for (i, c) in self
-      .changes
-      .range((Excluded(start_instant), Included(end_instant)))
-    {
-      changes.push((i, c));
-    }
-    changes
+  ) -> Box<dyn Iterator<Item=(Timestamp, &CacheChange)> + '_> {
+    Box::new(
+      self
+        .changes
+        .range((Excluded(start_instant), Included(end_instant)))
+        .map(|(i,c)| (*i,c)))
   }
 
   /// Removes and returns value if it was found
@@ -332,7 +286,6 @@ impl DDSHistoryCache {
 mod tests {
   use std::sync::{Arc, RwLock};
   use std::{thread};
-  use log::info;
 
   use super::DDSCache;
   use crate::{
@@ -405,16 +358,16 @@ mod tests {
           &(DDSTimestamp::now() - DDSDuration::from_secs(23)),
           &DDSTimestamp::now()
         )
-        .len(),
+        .count(),
       3
     );
-    info!(
-      "{:?}",
-      cache.read().unwrap().from_topic_get_changes_in_range(
-        topic_name,
-        &(DDSTimestamp::now() - DDSDuration::from_secs(23)),
-        &DDSTimestamp::now()
-      )
-    );
+    // info!(
+    //   "{:?}",
+    //   cache.read().unwrap().from_topic_get_changes_in_range(
+    //     topic_name,
+    //     &(DDSTimestamp::now() - DDSDuration::from_secs(23)),
+    //     &DDSTimestamp::now()
+    //   )
+    // );
   }
 }
