@@ -389,7 +389,7 @@ impl GUID {
   /// Generates GUID for specific entityId from current prefix
   pub fn from_prefix(self, entity_id: EntityId) -> GUID {
     GUID {
-      guidPrefix: self.guidPrefix.clone(),
+      guidPrefix: self.guidPrefix,
       entityId: entity_id,
     }
   }
@@ -420,9 +420,9 @@ pub(crate) struct GUIDData {
 impl GUIDData {
   pub fn from(guid: GUID, parameter_id: ParameterId) -> GUIDData {
     GUIDData {
-      parameter_id: parameter_id,
+      parameter_id,
       parameter_length: 16,
-      guid: guid,
+      guid,
     }
   }
 }
